// FluxPartitioning.cpp : Cve[V t@C
//

#include "stdafx.h"
#include "FluxAnalysisTool.h"
#include "FluxPartitioningDlg.h"

#pragma warning( disable : 4786 )

#include "BrowseFolderDialog.h"
#include "MeanTerm.h"
#include <vector>
#include<math.h>

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFluxPartitioning _CAO


CFluxPartitioningDlg::CFluxPartitioningDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFluxPartitioningDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFluxPartitioningDlg)
	m_strInputFile = _T("");
	m_strOutputFile = _T("");
	m_iRunPerDay = 48;
	m_iRespirationTerm = 29;
	m_iPhotosynthesisTerm = 15;
	m_dUstar_threshold = 0.2;
	m_bSetMaximumB = FALSE;
	m_dMaximumB = 0.005;
	m_dConvexity = 0.9;
	m_bUseNonRectangular = TRUE;
	m_dNighttimeRadiation = 10.0;
	m_bUseLT_Equation = TRUE;
	m_bUseFixedData = FALSE;
	m_strFixedDataFile = _T("");
	m_bUsePeriod = FALSE;
	m_strInputFileApply = _T("");
	m_strOutputFileApply = _T("");
	m_iMaximumForLinearRegression = 3;
	m_strInputRE = _T("");
	m_strInputGPP = _T("");
	m_bUseLinearRegression = TRUE;
	m_iGapFillingTerm = 15;
	m_bMeanDiurnalVariation = TRUE;
	m_bNonLinearRegression = TRUE;
	m_bMovingAverage = TRUE;
	m_bPhotosynthesis = TRUE;
	m_bRespiration = TRUE;
	m_iQCMaximumLoop = 1;
	m_dQCThresholdSigma = 6.0;
	m_dMaximumQ10 = 450.0 * 8.314472;
	m_dReferenceT_RE = 10.;
	m_iBootstrapNum = 100;
	m_bUseVanGorsel = FALSE;
	m_iSunsetRunVanGorsel = 6;
	m_bEliminateNegativeNEE = FALSE;
	//}}AFX_DATA_INIT

	m_pThread = NULL;
	m_bIsThread = FALSE;
	m_bBootstraping = FALSE;
}


void CFluxPartitioningDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFluxPartitioningDlg)
	DDX_Control(pDX, IDC_PROGRESS, m_ctlProgress);
	DDX_Text(pDX, IDC_EDIT_INPUT_FILE, m_strInputFile);
	DDX_Text(pDX, IDC_EDIT_OUTPUT_FILE, m_strOutputFile);
	DDX_Text(pDX, IDC_EDIT_RUN_PER_DAY, m_iRunPerDay);
	DDV_MinMaxUInt(pDX, m_iRunPerDay, 0, 500);
	DDX_Text(pDX, IDC_EDIT_RE_TERM, m_iRespirationTerm);
	DDV_MinMaxUInt(pDX, m_iRespirationTerm, 1, 1000000);
	DDX_Text(pDX, IDC_EDIT_GPP_TERM, m_iPhotosynthesisTerm);
	DDV_MinMaxUInt(pDX, m_iPhotosynthesisTerm, 1, 1000000);
	DDX_Text(pDX, IDC_EDIT_U_STAR, m_dUstar_threshold);
	DDV_MinMaxDouble(pDX, m_dUstar_threshold, 0., 100.);
	DDX_Check(pDX, IDC_CHECK_MAX_B, m_bSetMaximumB);
	DDX_Text(pDX, IDC_EDIT_MAX_B, m_dMaximumB);
	DDX_Text(pDX, IDC_EDIT_CONVEXITY, m_dConvexity);
	DDV_MinMaxDouble(pDX, m_dConvexity, 0., 1.);
	DDX_Check(pDX, IDC_CHECK_NON_RECTANGULAR, m_bUseNonRectangular);
	DDX_Text(pDX, IDC_EDIT_NIGHT_RADIATION, m_dNighttimeRadiation);
	DDX_Check(pDX, IDC_CHECK_LT_EQUATION, m_bUseLT_Equation);
	DDX_Check(pDX, IDC_CHECK_FIXED_DATA, m_bUseFixedData);
	DDX_Text(pDX, IDC_EDIT_INPUT_FIXED_DATA, m_strFixedDataFile);
	DDX_Check(pDX, IDC_CHECK_PERIOD, m_bUsePeriod);
	DDX_Text(pDX, IDC_EDIT_INPUT_APLLY, m_strInputFileApply);
	DDX_Text(pDX, IDC_EDIT_OUTPUT_APLLY, m_strOutputFileApply);
	DDX_Text(pDX, IDC_EDIT_MAXIMUM_REGRESSION, m_iMaximumForLinearRegression);
	DDX_Text(pDX, IDC_EDIT_INPUT_RE, m_strInputRE);
	DDX_Text(pDX, IDC_EDIT_INPUT_GPP, m_strInputGPP);
	DDX_Check(pDX, IDC_CHECK_USE_REGRESSION, m_bUseLinearRegression);
	DDX_Text(pDX, IDC_EDIT_GAP_TERM, m_iGapFillingTerm);
	DDV_MinMaxUInt(pDX, m_iGapFillingTerm, 1, 1000000);
	DDX_Check(pDX, IDC_CHECK_MDV, m_bMeanDiurnalVariation);
	DDX_Check(pDX, IDC_CHECK_NLR, m_bNonLinearRegression);
	DDX_Check(pDX, IDC_CHECK_MOVING_WD, m_bMovingAverage);
	DDX_Check(pDX, IDC_CHECK_PHOTOSYNTHESIS, m_bPhotosynthesis);
	DDX_Check(pDX, IDC_CHECK_RESPIRATION, m_bRespiration);
	DDX_Text(pDX, IDC_EDIT_QC_LOOP, m_iQCMaximumLoop);
	DDV_MinMaxInt(pDX, m_iQCMaximumLoop, 0, 1000);
	DDX_Text(pDX, IDC_EDIT_QC_SIGMA, m_dQCThresholdSigma);
	DDV_MinMaxDouble(pDX, m_dQCThresholdSigma, 0., 10.);
	DDX_Text(pDX, IDC_EDIT_MAX_Q10, m_dMaximumQ10);
	DDX_Text(pDX, IDC_EDIT_REF_T_RE, m_dReferenceT_RE);
	DDX_Text(pDX, IDC_EDIT_BOOTSTRAP_NUM, m_iBootstrapNum);
	DDV_MinMaxInt(pDX, m_iBootstrapNum, 3, 2000);
	DDX_Check(pDX, IDC_CHECK_VAN_GORSEL, m_bUseVanGorsel);
	DDX_Text(pDX, IDC_EDIT_VAN_GORSEL, m_iSunsetRunVanGorsel);
	DDV_MinMaxUInt(pDX, m_iSunsetRunVanGorsel, 1, 96);
	DDX_Check(pDX, IDC_CHECK_OMIT_NEGATIVE_NEE, m_bEliminateNegativeNEE);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFluxPartitioningDlg, CDialog)
	//{{AFX_MSG_MAP(CFluxPartitioningDlg)
	ON_BN_CLICKED(IDC_BUTTON_INPUT_FILE, OnButtonInputFile)
	ON_BN_CLICKED(IDC_BUTTON_OUTPUT_FILE, OnButtonOutputFile)
	ON_EN_KILLFOCUS(IDC_EDIT_INPUT_FILE, OnKillfocus)
	ON_BN_CLICKED(IDC_BTN_DETERMINE, OnBtnDetermine)
	ON_BN_CLICKED(IDC_CHECK_NON_RECTANGULAR, OnCheckNonRectangular)
	ON_BN_CLICKED(IDC_CHECK_MAX_B, OnCheckMaxB)
	ON_BN_CLICKED(IDC_CHECK_FIXED_DATA, OnCheckFixedData)
	ON_BN_CLICKED(IDC_BUTTON_FIXED_DATA, OnButtonFixedData)
	ON_BN_CLICKED(IDC_BUTTON_INPUT_APPLY, OnButtonInputApply)
	ON_BN_CLICKED(IDC_CHECK_LT_EQUATION, OnCheckLtEquation)
	ON_BN_CLICKED(IDC_CHECK_PERIOD, OnCheckPeriod)
	ON_BN_CLICKED(IDC_CHECK_USE_REGRESSION, OnCheckUseRegression)
	ON_BN_CLICKED(IDC_BUTTON_OUTPUT_APPLY, OnButtonOutputApply)
	ON_BN_CLICKED(IDC_BUTTON_INPUT_GPP, OnButtonInputGpp)
	ON_BN_CLICKED(IDC_BUTTON_INPUT_RE, OnButtonInputRe)
	ON_BN_CLICKED(IDC_BTN_APPLY, OnBtnApply)
	ON_BN_CLICKED(IDC_CHECK_MDV, OnCheckMdv)
	ON_BN_CLICKED(IDC_CHECK_NLR, OnCheckNlr)
	ON_BN_CLICKED(IDC_CHECK_RESPIRATION, OnCheckRespiration)
	ON_BN_CLICKED(IDC_CHECK_PHOTOSYNTHESIS, OnCheckPhotosynthesis)
	ON_BN_CLICKED(IDC_CHECK_MOVING_WD, OnCheckMovingWd)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BTN_LAOD, OnBtnLaod)
	ON_BN_CLICKED(IDC_BTN_SAVE, OnBtnSave)
	ON_BN_CLICKED(IDC_BTN_UNCERTAIN, OnBtnUncertain)
	ON_BN_CLICKED(IDC_CHECK_VAN_GORSEL, OnCheckVanGorsel)
	ON_EN_KILLFOCUS(IDC_EDIT_OUTPUT_FILE, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_RUN_PER_DAY, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_U_STAR, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_GPP_TERM, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_RE_TERM, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_MAX_B, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_CONVEXITY, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_INPUT_FIXED_DATA, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_INPUT_APLLY, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_OUTPUT_APLLY, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_INPUT_GPP, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_INPUT_RE, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_MAXIMUM_REGRESSION, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_GAP_TERM, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_QC_SIGMA, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_QC_LOOP, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_MAX_Q10, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_BOOTSTRAP_NUM, OnKillfocus)
	ON_EN_KILLFOCUS(IDC_EDIT_VAN_GORSEL, OnKillfocus)
	ON_BN_CLICKED(IDC_CHECK_OMIT_NEGATIVE_NEE, OnCheckOmitNegativeNee)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFluxPartitioning bZ[W nh

BOOL CFluxPartitioningDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: ̈ʒuɏ̕⑫ǉĂ
	GetDlgItem(IDC_EDIT_MAX_B)->EnableWindow(FALSE);
	GetDlgItem(IDC_STATIC_NON_RECT)->EnableWindow(TRUE);
	GetDlgItem(IDC_EDIT_CONVEXITY)->EnableWindow(TRUE);

	GetDlgItem(IDC_STATIC_FIXED_DATA)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_INPUT_FIXED_DATA)->EnableWindow(FALSE);
	GetDlgItem(IDC_BUTTON_FIXED_DATA)->EnableWindow(FALSE);

	GetDlgItem(IDC_EDIT_VAN_GORSEL)->EnableWindow(FALSE);

	CWinApp* pApp = AfxGetApp();
	HICON hFile = pApp->LoadIcon( IDR_MAINFRAME );
	SetIcon(hFile, TRUE);	
	SetIcon(hFile, FALSE);	
	
	return TRUE;  // Rg[ɃtH[JXݒ肵ȂƂA߂l TRUE ƂȂ܂
	              // O: OCX vpeB y[W̖߂l FALSE ƂȂ܂
}

BOOL CFluxPartitioningDlg::DestroyWindow() 
{
	// TODO: ̈ʒuɌŗL̏ǉ邩A܂͊{NXĂяoĂ
	if( m_bIsThread ){
		if ( ::WaitForSingleObject(m_pThread->m_hThread, 2000) ==  WAIT_TIMEOUT ){
			TerminateThread(m_pThread->m_hThread, 1);			
		}

		delete m_pThread;
		m_pThread = NULL;
	}
	
	return CDialog::DestroyWindow();
}

void CFluxPartitioningDlg::OnClose() 
{
	// TODO: ̈ʒuɃbZ[W nhp̃R[hǉ邩܂̓ftHg̏ĂяoĂ
	if( m_bIsThread ){
		if(AfxMessageBox("Although the calculation has been operated, do you terminate the process?", MB_OKCANCEL) == IDCANCEL){
			return;
		}
	}
	
	CDialog::OnClose();
}

void CFluxPartitioningDlg::OnCheckNonRectangular() 
{
	UpdateData(TRUE);

	if(m_bUseNonRectangular == TRUE){		
		GetDlgItem(IDC_STATIC_NON_RECT)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_CONVEXITY)->EnableWindow(TRUE);

	}else{
		GetDlgItem(IDC_STATIC_NON_RECT)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_CONVEXITY)->EnableWindow(FALSE);
	}	
	UpdateData(FALSE);
}

void CFluxPartitioningDlg::OnCheckMaxB() 
{
	// TODO: ̈ʒuɃRg[ʒmnhp̃R[hǉĂ
	UpdateData(TRUE);

	if(m_bSetMaximumB == TRUE){		
		GetDlgItem(IDC_EDIT_MAX_B)->EnableWindow(TRUE);

	}else{
		GetDlgItem(IDC_EDIT_MAX_B)->EnableWindow(FALSE);
	}	
	UpdateData(FALSE);
}

void CFluxPartitioningDlg::OnCheckFixedData() 
{
	UpdateData(TRUE);

	if(m_bUseFixedData == TRUE){		
		GetDlgItem(IDC_STATIC_FIXED_DATA)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_INPUT_FIXED_DATA)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_FIXED_DATA)->EnableWindow(TRUE);

	}else{
		GetDlgItem(IDC_STATIC_FIXED_DATA)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_INPUT_FIXED_DATA)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_FIXED_DATA)->EnableWindow(FALSE);
	}	
	UpdateData(FALSE);
}

void CFluxPartitioningDlg::OnButtonInputFile() 
{
	CFileDialog dlg(TRUE, NULL, NULL, 
		OFN_FILEMUSTEXIST| OFN_HIDEREADONLY| OFN_LONGNAMES,
		"csv  t@C   (*.csv)|*.csv|"
		"SĂ t@C   (*.*)|*.*|",
		NULL);

	if(dlg.DoModal() == IDOK){
		m_strInputFile = dlg.GetPathName();
		m_strInputTitle = dlg.GetFileTitle();
		if(m_strOutputFile == ""){
			m_strOutputFile = m_strInputFile.Mid(0, m_strInputFile.ReverseFind('\\'));
		}
		UpdateData(FALSE);
	}
}

void CFluxPartitioningDlg::OnButtonFixedData() 
{
	CFileDialog dlg(TRUE, NULL, NULL, 
		OFN_FILEMUSTEXIST| OFN_HIDEREADONLY| OFN_LONGNAMES,
		"csv  t@C   (*.csv)|*.csv|"
		"SĂ t@C   (*.*)|*.*|",
		NULL);

	if(dlg.DoModal() == IDOK){
		m_strFixedDataFile = dlg.GetPathName();
		UpdateData(FALSE);
	}
}

void CFluxPartitioningDlg::OnButtonOutputFile() 
{
	CBrowseFolderDialog dlg;
	UpdateData(TRUE);
	
	if(dlg.DoModal() == IDOK){
		m_strOutputFile = dlg.GetPathName();
		UpdateData(FALSE);
	}
}

void CFluxPartitioningDlg::OnButtonInputApply() 
{
	CFileDialog dlg(TRUE, NULL, NULL, 
		OFN_FILEMUSTEXIST| OFN_HIDEREADONLY| OFN_LONGNAMES,
		"csv  t@C   (*.csv)|*.csv|"
		"SĂ t@C   (*.*)|*.*|",
		NULL);

	if(dlg.DoModal() == IDOK){
		m_strInputFileApply = dlg.GetPathName();
		m_strInputTitleApply = dlg.GetFileTitle();

		if(m_strOutputFileApply == ""){
			m_strOutputFileApply = m_strInputFileApply.Mid(0, m_strInputFileApply.ReverseFind('\\'));
		}

		UpdateData(FALSE);
	}
}


void CFluxPartitioningDlg::OnButtonOutputApply() 
{
	CBrowseFolderDialog dlg;
	UpdateData(TRUE);
	
	if(dlg.DoModal() == IDOK){
		m_strOutputFileApply = dlg.GetPathName();
		UpdateData(FALSE);
	}
}

void CFluxPartitioningDlg::OnButtonInputGpp() 
{
	CFileDialog dlg(TRUE, NULL, NULL, 
		OFN_FILEMUSTEXIST| OFN_HIDEREADONLY| OFN_LONGNAMES,
		"csv  t@C   (*.csv)|*.csv|"
		"SĂ t@C   (*.*)|*.*|",
		NULL);

	if(dlg.DoModal() == IDOK){
		m_strInputGPP = dlg.GetPathName();
		UpdateData(FALSE);
	}
}

void CFluxPartitioningDlg::OnButtonInputRe() 
{
	CFileDialog dlg(TRUE, NULL, NULL, 
		OFN_FILEMUSTEXIST| OFN_HIDEREADONLY| OFN_LONGNAMES,
		"csv  t@C   (*.csv)|*.csv|"
		"SĂ t@C   (*.*)|*.*|",
		NULL);

	if(dlg.DoModal() == IDOK){
		m_strInputRE = dlg.GetPathName();
		UpdateData(FALSE);
	}
}

void CFluxPartitioningDlg::OnKillfocus() 
{
	UpdateData(TRUE);	
}

void CFluxPartitioningDlg::OnCheckLtEquation() 
{
	UpdateData(TRUE);

	if(m_bUseLT_Equation == FALSE && m_dMaximumQ10 == 450.0 * 8.314472){
		m_dMaximumQ10 = 5.0;
		GetDlgItem(IDC_EDIT_REF_T_RE)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_REF_T_RE)->EnableWindow(FALSE);
	}
	if(m_bUseLT_Equation == TRUE && m_dMaximumQ10 == 5.0){
		m_dMaximumQ10 = 450.0 * 8.314472;
		GetDlgItem(IDC_EDIT_REF_T_RE)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_REF_T_RE)->EnableWindow(TRUE);
	}

	UpdateData(FALSE);
}

void CFluxPartitioningDlg::OnCheckPeriod() 
{
	UpdateData(TRUE);
}

void CFluxPartitioningDlg::OnCheckUseRegression() 
{
	UpdateData(TRUE);

	if(m_bUseLinearRegression == TRUE){
		GetDlgItem(IDC_EDIT_MAXIMUM_REGRESSION)->EnableWindow(TRUE);
	}
	else{
		GetDlgItem(IDC_EDIT_MAXIMUM_REGRESSION)->EnableWindow(FALSE);
	}
}

void CFluxPartitioningDlg::OnCheckPhotosynthesis() 
{
	UpdateData(TRUE);

	if(m_bPhotosynthesis == TRUE){
		GetDlgItem(IDC_STATIC_PHOTO_TERM)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_GPP_TERM)->EnableWindow(TRUE);
		GetDlgItem(IDC_CHECK_MAX_B)->EnableWindow(TRUE);
		if(m_bSetMaximumB == TRUE) GetDlgItem(IDC_EDIT_MAX_B)->EnableWindow(TRUE);
		GetDlgItem(IDC_CHECK_NON_RECTANGULAR)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_NON_RECT)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_CONVEXITY)->EnableWindow(TRUE);
	}
	else{
		GetDlgItem(IDC_STATIC_PHOTO_TERM)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_GPP_TERM)->EnableWindow(FALSE);
		GetDlgItem(IDC_CHECK_MAX_B)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_MAX_B)->EnableWindow(FALSE);
		GetDlgItem(IDC_CHECK_NON_RECTANGULAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_NON_RECT)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_CONVEXITY)->EnableWindow(FALSE);
	}
}

void CFluxPartitioningDlg::OnCheckRespiration() 
{
	UpdateData(TRUE);

	if(m_bRespiration == TRUE){
		GetDlgItem(IDC_CHECK_LT_EQUATION)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_RE_TERM)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_RE_TERM)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_MAX_Q10)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_MAX_Q10)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_REF_T_RE)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_REF_T_RE)->EnableWindow(TRUE);
	}
	else{
		GetDlgItem(IDC_CHECK_LT_EQUATION)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_RE_TERM)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_RE_TERM)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MAX_Q10)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_MAX_Q10)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_REF_T_RE)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_REF_T_RE)->EnableWindow(FALSE);
	}
}

void CFluxPartitioningDlg::OnCheckMdv() 
{
	UpdateData(TRUE);

	if(m_bMeanDiurnalVariation == FALSE){
		// GetDlgItem(IDC_STATIC_GAP_TERM)->EnableWindow(FALSE);
		// GetDlgItem(IDC_EDIT_GAP_TERM)->EnableWindow(FALSE);
		GetDlgItem(IDC_CHECK_USE_REGRESSION)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_MAXIMUM_REGRESSION)->EnableWindow(FALSE);
	}
	else{
		// GetDlgItem(IDC_STATIC_GAP_TERM)->EnableWindow(TRUE);
		// GetDlgItem(IDC_EDIT_GAP_TERM)->EnableWindow(TRUE);
		GetDlgItem(IDC_CHECK_USE_REGRESSION)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_MAXIMUM_REGRESSION)->EnableWindow(TRUE);
	}
}

void CFluxPartitioningDlg::OnCheckNlr() 
{
	UpdateData(TRUE);

	if(m_bNonLinearRegression == TRUE){
		GetDlgItem(IDC_STATIC_GPP)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_INPUT_GPP)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_INPUT_GPP)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_RE)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_INPUT_RE)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_INPUT_RE)->EnableWindow(TRUE);
	}
	else{
		GetDlgItem(IDC_STATIC_GPP)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_INPUT_GPP)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_INPUT_GPP)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_RE)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_INPUT_RE)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_INPUT_RE)->EnableWindow(FALSE);
	}
}

void CFluxPartitioningDlg::OnCheckVanGorsel() 
{
	UpdateData(TRUE);

	if(m_bUseVanGorsel == TRUE){
		GetDlgItem(IDC_EDIT_VAN_GORSEL)->EnableWindow(TRUE);
	}
	else{
		GetDlgItem(IDC_EDIT_VAN_GORSEL)->EnableWindow(FALSE);
	}
}

void CFluxPartitioningDlg::OnCheckMovingWd() 
{
	UpdateData(TRUE);
}

void CFluxPartitioningDlg::OnCheckOmitNegativeNee() 
{
	UpdateData(TRUE);
}

void CFluxPartitioningDlg::OnBtnDetermine() 
{
	BOOL bIsDebug = FALSE;
	#ifdef _DEBUG
	bIsDebug = TRUE;
	#endif

	CString strErrorMessage;
	if(bIsDebug){
		if(Determine_Photosynthesis_Respiration_Parameters(strErrorMessage) == TRUE){
			AfxMessageBox("The calculation has been successfully done !");
		}
		else{
			AfxMessageBox(strErrorMessage);
		}
	}
	else{
		// Using Malti Thread
		if(!m_bIsThread){
			m_pThread = AfxBeginThread(OnDetermineThread,(LPVOID)this);
			m_pThread->m_bAutoDelete = FALSE;
			m_pThread->ResumeThread();
		}
		else{
			AfxMessageBox("Error : Thread is busy !");
		}
	}
}

UINT CFluxPartitioningDlg::OnDetermineThread(LPVOID pParam)
{
	CFluxPartitioningDlg* pDlg = static_cast<CFluxPartitioningDlg*>(pParam);

 	pDlg->m_bIsThread = TRUE;
	if(pDlg->Determine_Photosynthesis_Respiration_Parameters(pDlg->m_strErrorMessageForThread) == TRUE){
		AfxMessageBox("The calculation has been successfully done !");
	}
	else{
		if(pDlg->m_strErrorMessageForThread != ""){
			AfxMessageBox(pDlg->m_strErrorMessageForThread);
		}
	}

	pDlg->m_bIsThread = FALSE;
	return 0;
}

BOOL CFluxPartitioningDlg::Determine_Photosynthesis_Respiration_Parameters(CString& strErrorMessage) 
{
	// TODO: ̈ʒuɃRg[ʒmnhp̃R[hǉĂ
	UpdateData(TRUE);

	BeginWaitCursor();

	///////////////////////////////////////////
	// eLXg{bNX̓͂̃`FbN
	///////////////////////////////////////////
	if(ErrorCheck(strErrorMessage) == FALSE){
		return FALSE;
	}

	// ***************************************************************
	// ݒt@C̏o
	// ***************************************************************
	CString strOutSettingFile = m_strInputFile;
	strOutSettingFile = strOutSettingFile.Left(strOutSettingFile.ReverseFind('.')) + ".determine.set.txt";
	strOutSettingFile = m_strOutputFile + "\\" + strOutSettingFile.Mid(strOutSettingFile.ReverseFind('\\') + 1);
	SaveSetting(strOutSettingFile);

	///////////////////////////////////////////
	// Œp^̓ǂݍ
	///////////////////////////////////////////	
	if(m_bUseFixedData){
		if(ReadFixedParamterFile() == FALSE) return FALSE;
	}

	// **************************************************
	// vOXo[̏
	// **************************************************
	int iProgressRange = 11000;
	m_ctlProgress.SetRange(0, iProgressRange);	// Inisilize Progress bar
	m_ctlProgress.SetPos(200);
	
	///////////////////////////////////////////
	// ̓t@C̓ǂݍ
	///////////////////////////////////////////	
	vector<CMeanTerm*> aryMeanTermGPP;
	vector<CMeanTerm*> aryMeanTermRE;
	CString strHeaderLine;
	BOOL bExistHeader = TRUE;
	if(m_bMovingAverage){
		if(ReadFile_with_MovingWindow(m_strInputFile, aryMeanTermGPP, aryMeanTermRE, m_iRunPerDay, m_iPhotosynthesisTerm, m_iRespirationTerm, strHeaderLine) == FALSE){
			// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
			strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
			return FALSE;
		}
	}
	else{
		if(ReadFile_with_BlockWindow(m_strInputFile, aryMeanTermGPP, aryMeanTermRE, m_iRunPerDay, m_iPhotosynthesisTerm, m_iRespirationTerm, strHeaderLine) == FALSE){
			// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
			strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
			return FALSE;
		}
	}
	m_ctlProgress.SetPos(1000);

	///////////////////////////////////////////
	// ֘Ãp^Zo
	///////////////////////////////////////////	
	if(m_bPhotosynthesis == TRUE){
		DeterminePhotosynthesisParameters(aryMeanTermGPP, strHeaderLine, m_ctlProgress);
	}
	else{
		// mۂ̈̊J
		for(int i = 0; i < aryMeanTermGPP.size(); i++){
			aryMeanTermGPP[i]->ReleaseDataset();
			delete aryMeanTermGPP[i];
		}
	}

	///////////////////////////////////////////
	// ċz֘Ãp^Zo
	///////////////////////////////////////////	
	if(m_bRespiration == TRUE){
		DetermineRespirationParameters(aryMeanTermRE, strHeaderLine, m_ctlProgress);
	}
	else{
		// mۂ̈̊J
		for(int i = 0; i < aryMeanTermRE.size(); i++){
			aryMeanTermRE[i]->ReleaseDataset();
			delete aryMeanTermRE[i];
		}
	}

	//AfxMessageBox("The calculation has been successfully done!");
	strErrorMessage = "The calculation has been successfully done!";

	return TRUE;
}

BOOL CFluxPartitioningDlg::ErrorCheck(CString& strErrorMessage)
{
	if(m_strInputFile == ""){
		// AfxMessageBox ("̓t@Cw肵ĂB");
		strErrorMessage = "Please select the input file.";
		return FALSE;
	}
	if(m_strOutputFile == ""){
		// AfxMessageBox ("o̓t@Cw肵ĂB");
		strErrorMessage = "Please select the output directory.";
		return FALSE;
	}
	if(m_iRunPerDay <= 0){
		// AfxMessageBox ("Run per Day ͐̐ɂĉB");
		strErrorMessage = "The Run per day should be positive value.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_RUN_PER_DAY));
		return FALSE;
	}
	if(m_iPhotosynthesisTerm < 1){
		// AfxMessageBox ("The photosynthesis term should be greater than 1.");
		strErrorMessage = "The photosynthesis term should be greater than 1.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_GPP_TERM));
		return FALSE;
	}
	if(m_iRespirationTerm < 1){
		// AfxMessageBox ("The respiration term should be greater than 1.");
		strErrorMessage = "TThe respiration term should be greater than 1.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_RE_TERM));
		return FALSE;
	}
	if(m_bMovingAverage && m_iPhotosynthesisTerm % 2 == 0){
		// AfxMessageBox ("TThe respiration term should be greater than 1.");
		strErrorMessage = "The photosynthesis term should be odd number.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_GPP_TERM));
		return FALSE;
	}
	if(m_bMovingAverage && m_iRespirationTerm % 2 == 0){
		// AfxMessageBox ("The respiration term should be odd number.");
		strErrorMessage = "The respiration term should be odd number.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_RE_TERM));
		return FALSE;
	}
	if(m_bSetMaximumB == TRUE && m_dMaximumB <= 0.0){
		// AfxMessageBox ("Analytical Term ͐̐ɂĉB");
		strErrorMessage = "The Analytical term should be positive value.";
		return FALSE;
	}
	if(m_bUseNonRectangular == TRUE && m_dConvexity <= 0.0 || m_dConvexity > 1.0){
		// AfxMessageBox ("Convexity  0`1 ̂ɂĉB");
		strErrorMessage = "The convexity should be between 0 and 1.";
		return FALSE;
	}
	if(m_bUseFixedData == TRUE && m_strFixedDataFile <= ""){
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_INPUT_FIXED_DATA));
		// AfxMessageBox ("Please select the fixed paramter file when the use fixed parameter set is checked.");
		strErrorMessage = "Please select the fixed paramter file when the use fixed parameter set is checked.";
		return FALSE;
	}
	if(m_bPhotosynthesis == FALSE && m_bRespiration == FALSE){
		return FALSE;
	}

	return TRUE;
}

void CFluxPartitioningDlg::DeterminePhotosynthesisParameters(std::vector<CMeanTerm*>& aryMeanTermGPP, const CString& strHeaderLine, CProgressCtrl& ctlProgress)
{

	///////////////////////////////////////////
	// ŏ@ɂ[Ȑ̍쐬
	///////////////////////////////////////////	
	int iUstarColumn = 0;
	int iRadiationColumn = 4;
	int iPhotosynthesisColumn = 3;
	int iRadiationFilledColumn = 7;
	int iTemperatureFilledColumn = 8;
	double dMaximum, dMinimum, dAverage;
	BOOL bUseNegativeValues = TRUE;
	BOOL bCalculateR0 = TRUE;
	int iReplaced = -1, iLoopCount = 0;
	CPhotosynthesisParameters oldPrm;

	vector<int> aryUstarCorrectionColumn;
	aryUstarCorrectionColumn.push_back(iPhotosynthesisColumn);

	///////////////////////////////////////////
	// t@CEtH_쐬Aʂۑ
	///////////////////////////////////////////	
	CString strOutputFileName;
	m_strInputTitle = m_strInputFile.Mid(m_strInputFile.ReverseFind('\\') + 1);
	m_strInputTitle = m_strInputTitle.Mid(0, m_strInputTitle.ReverseFind('.'));
	if(m_bUseVanGorsel) m_strInputTitle += ".vG";

	if(m_bUseNonRectangular == 0){
		if(m_bMovingAverage)
			strOutputFileName.Format("%s\\%s_mw%d_L-P-curve.csv", m_strOutputFile, m_strInputTitle, m_iPhotosynthesisTerm);
		else
			strOutputFileName.Format("%s\\%s_bw%d_L-P-curve.csv", m_strOutputFile, m_strInputTitle, m_iPhotosynthesisTerm);
	}
	else{
		if(m_bMovingAverage)
			strOutputFileName.Format("%s\\%s_mw%d_nL-P-curve_%.2lf.csv", m_strOutputFile, m_strInputTitle, m_iPhotosynthesisTerm, m_dConvexity);
		else
			strOutputFileName.Format("%s\\%s_bw%d_nL-P-curve_%.2lf.csv", m_strOutputFile, m_strInputTitle, m_iPhotosynthesisTerm, m_dConvexity);
	}
	if(m_bBootstraping){
		CString strTempFileName = strOutputFileName.Mid(0, strOutputFileName.ReverseFind('.'));
		strOutputFileName.Format("%s_bootstrap.csv", strTempFileName);
	}

	try{
		CStdioFile file(strOutputFileName, CFile::modeCreate | CFile::modeWrite);

		// sڂɗLq
		CString strLine;
		if(m_bBootstraping == FALSE){
			strLine.Format("Date, Amax, b, Rd, RMSE, R, Tmax, Tmin, Tave, Data Number, iCount, Negative Values:%d, Non Rectangular:%d, Convexity:%lf\n", bUseNegativeValues, m_bUseNonRectangular, m_dConvexity);
		}
		else{
			strLine.Format("Date, Amax_ave, b_ave, Rd_ave, Amax_stdev, b_stdev, Rd_stdev\n");
		}
		file.WriteString(strLine);

		double dProgressSize = 4000.0 / aryMeanTermGPP.size();
		for(int i = 0; i < aryMeanTermGPP.size(); i++){
			// ԂƂɕf[^AꍀڂƂ̃f[^Zbgɐ؂蕪	
			CString strTemp = strHeaderLine;
			if(aryMeanTermGPP[i]->SetDataset(strTemp.Replace(',','@')) == FALSE){
				return;
			}
			aryMeanTermGPP[i]->SetHeader(strHeaderLine);

			// eliminate negative nighttime NEE
			if(m_bEliminateNegativeNEE){
				if(aryMeanTermGPP[i]->Eliminate_nighttime_negative(aryUstarCorrectionColumn, m_dNighttimeRadiation, iPhotosynthesisColumn, iRadiationFilledColumn) == FALSE){
					return;
				}
			}

			// u* filtering
			if(aryMeanTermGPP[i]->Ustar_filtering(aryUstarCorrectionColumn, m_dNighttimeRadiation, m_dUstar_threshold, iUstarColumn, iRadiationFilledColumn) == FALSE){
				return;
			}

			// van Gorsel filtering added 2014.06.24
			if(m_bUseVanGorsel){
				aryMeanTermGPP[i]->VanGorsel_filtering(m_dNighttimeRadiation, m_iSunsetRunVanGorsel, iPhotosynthesisColumn, iRadiationFilledColumn);
			}

			// poȐ̌
			while(iReplaced != 0){
				// poȐ̌
				if(m_bBootstraping == FALSE){
					if(aryMeanTermGPP[i]->DetermineLightPhotosynthesisCurve(m_bSetMaximumB, m_dMaximumB, m_dConvexity, m_dNighttimeRadiation, bCalculateR0, bUseNegativeValues, m_bUseNonRectangular, iRadiationColumn, iPhotosynthesisColumn, m_bUseFixedData, m_aryFixedPhotosynthesisParameters[i]) == FALSE){
						return;
					}
				}
				else{
					if(aryMeanTermGPP[i]->DetermineLightPhotosynthesisCurve_bootstrap(m_bSetMaximumB, m_dMaximumB, m_dConvexity, m_dNighttimeRadiation, bCalculateR0, bUseNegativeValues, m_bUseNonRectangular, iRadiationColumn, iPhotosynthesisColumn, m_bUseFixedData, m_aryFixedPhotosynthesisParameters[i], m_iBootstrapNum) == FALSE){
						return;
					}
				}
				break;		// ɊւẮAoutliner ̏͂Ȃ@iƕςɂȂj
				
				//if(m_dQCThresholdSigma == 0.0 || iLoopCount == m_iQCMaximumLoop) break;
				//if(iLoopCount != 0 && oldPrm.m_iDataNum / 2 > aryMeanTermGPP[i]->m_PhotosynthesisParameters.m_iDataNum) break;

				//if(iLoopCount == 0){
				//	oldPrm = aryMeanTermGPP[i]->m_PhotosynthesisParameters;
				//}
				//iReplaced = aryMeanTermGPP[i]->QC_PhotosynthesisData(m_dQCThresholdSigma, bUseNegativeValues, iRadiationColumn, iPhotosynthesisColumn);
				//iLoopCount++;
			}
			
			//if(iLoopCount != 0 && oldPrm.m_dMaximumPhotosynthesis != aryMeanTermGPP[i]->s_dErrorValue && aryMeanTermGPP[i]->m_PhotosynthesisParameters.m_dMaximumPhotosynthesis == aryMeanTermGPP[i]->s_dErrorValue){
			//	aryMeanTermGPP[i]->m_PhotosynthesisParameters = oldPrm;
			//}
			//iReplaced = -1;
			//iLoopCount = 0;

			if(m_bBootstraping == FALSE){
				// ōEŒEϋČ
				aryMeanTermGPP[i]->GetMaximumMinimuAverage(iTemperatureFilledColumn, dMaximum, dMinimum, dAverage);

				// f[^̏o
				strLine = aryMeanTermGPP[i]->GetOutputLine_LightPhotosynthesis(bUseNegativeValues, dMaximum, dMinimum, dAverage);
				file.WriteString(strLine);

				ctlProgress.SetPos(1000 + dProgressSize * i);
			}
			else{
				// bootstrap p^̕ρAW΍vZ
				CPhotosynthesisParameters cPhoto = aryMeanTermGPP[i]->Average_stdev_bootstrap_parameters_photosynthesis();

				// Bootstraping p^̎擾
				m_aryPhotosynthesisParametersBootstrap.clear();
				aryMeanTermGPP[i]->SetBootstrapPhosynthesisParameters(m_aryPhotosynthesisParametersBootstrap, cPhoto);

				// f[^̏o
				strLine = aryMeanTermGPP[i]->GetOutputLine_LightPhotosynthesis_bootstrap(bUseNegativeValues, cPhoto);
				file.WriteString(strLine);

				ctlProgress.SetPos(1000 / 2 + dProgressSize * i / 2);
			}

			// mۂ̈̊J
			if(m_bBootstraping == FALSE){
				aryMeanTermGPP[i]->ReleaseDataset();
				delete aryMeanTermGPP[i];
			}
		}

		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
	}

	if(m_bBootstraping == FALSE) aryMeanTermGPP.clear();
}

void CFluxPartitioningDlg::DetermineRespirationParameters(std::vector<CMeanTerm*>& aryMeanTermRE, const CString& strHeaderLine, CProgressCtrl& ctlProgress)
{
	///////////////////////////////////////////
	// Q10 ̌
	///////////////////////////////////////////	
	int iTemperatureColumn = 5;
	int iRadiationColumn = 4;
	int iRespirationColumn = 3;
	int iUstarColumn = 0;
	int iRadiationFilledColumn = 7;
	BOOL bUseNegativeValues = TRUE;
	int iReplaced = -1, iLoopCount = 0;
	CRespirationParameters oldPrm;
	vector<int> aryUstarCorrectionColumn;

	aryUstarCorrectionColumn.push_back(iRespirationColumn);

	///////////////////////////////////////////
	// t@CEtH_쐬Aʂۑ
	///////////////////////////////////////////	
	CString strOutputFileName;
	m_strInputTitle = m_strInputFile.Mid(m_strInputFile.ReverseFind('\\') + 1);
	m_strInputTitle = m_strInputTitle.Mid(0, m_strInputTitle.ReverseFind('.'));
	if(m_bUseVanGorsel) m_strInputTitle += ".vG";
	if(m_bUseLT_Equation == 0){
		if(m_bMovingAverage)
			strOutputFileName.Format("%s\\%s_mw%d_RE-curve.csv", m_strOutputFile, m_strInputTitle, m_iRespirationTerm);
		else
			strOutputFileName.Format("%s\\%s_bw%d_RE-curve.csv", m_strOutputFile, m_strInputTitle, m_iRespirationTerm);
	}
	else{
		if(m_bMovingAverage)
			strOutputFileName.Format("%s\\%s_mw%d_nRE-curve_LT.csv", m_strOutputFile, m_strInputTitle, m_iRespirationTerm);
		else
			strOutputFileName.Format("%s\\%s_bw%d_nRE-curve_LT.csv", m_strOutputFile, m_strInputTitle, m_iRespirationTerm);
	}	
	if(m_bBootstraping){
		CString strTempFileName = strOutputFileName.Mid(0, strOutputFileName.ReverseFind('.'));
		strOutputFileName.Format("%s_bootstrap.csv", strTempFileName);
	}

	try{
		CStdioFile file(strOutputFileName, CFile::modeCreate | CFile::modeWrite);

		// sڂɗLq
		CString strLine;
		if(m_bBootstraping == FALSE){
			if(m_bUseLT_Equation == FALSE){
				strLine.Format("Date, R0, Q10, Tref, RMSE, R, Tmax, Tmin, Tave, Data Number, Negative Values:%d, Lloyd and Taylor:%d, u* threshold:%lf, Nighttime radiation:%lf\n", bUseNegativeValues, m_bUseLT_Equation, m_dUstar_threshold, m_dNighttimeRadiation);
			}
			else{
				strLine.Format("Date, R10, E0, Tref, RMSE, R, Tmax, Tmin, Tave, Data Number, Negative Values:%d, Lloyd and Taylor:%d, u* threshold:%lf, Nighttime radiation:%lf\n", bUseNegativeValues, m_bUseLT_Equation, m_dUstar_threshold, m_dNighttimeRadiation);
			}
		}
		else{
			if(m_bUseLT_Equation == FALSE){
				strLine.Format("Date, R0_ave, Q10_ave, Tref_ave, R0_stdev, Q10_stdev, Tref_stdev\n");
			}
			else{
				strLine.Format("Date, R10_ave, E0_ave, Tref_ave, R10_stdev, E0_stdev, Tref_stdev\n");
			}
		}
		file.WriteString(strLine);

		double dProgressSize = 6000.0 / aryMeanTermRE.size();
		double dTmax, dTmin, dTave;
		BOOL bAnnual = FALSE;
		BOOL bUseFixedData = m_bUseFixedData;
		for(int i = 0; i < aryMeanTermRE.size(); i++){// ԂƂɕf[^AꍀڂƂ̃f[^Zbgɐ؂蕪	
			CString strTemp = strHeaderLine;
			if(aryMeanTermRE[i]->SetDataset(strTemp.Replace(',','@')) == FALSE){
				return;
			}
			aryMeanTermRE[i]->SetHeader(strHeaderLine);

			// Q10 ̌
			if(i == aryMeanTermRE.size() - 1){
				bAnnual = TRUE;
			}

			if(i == aryMeanTermRE.size() - 1) bUseFixedData = FALSE; // NԂ̊֌W߂ۂfix w肵Ȃ

			// eliminate negative nighttime NEE
			if(m_bEliminateNegativeNEE){
				if(aryMeanTermRE[i]->Eliminate_nighttime_negative(aryUstarCorrectionColumn, m_dNighttimeRadiation, iRespirationColumn, iRadiationFilledColumn) == FALSE){
					return;
				}
			}

			// u* filtering
			if(aryMeanTermRE[i]->Ustar_filtering(aryUstarCorrectionColumn, m_dNighttimeRadiation, m_dUstar_threshold, iUstarColumn, iRadiationFilledColumn) == FALSE){
				return;
			}

			// van Gorsel filtering added 2014.06.24
			if(m_bUseVanGorsel){
				aryMeanTermRE[i]->VanGorsel_filtering(m_dNighttimeRadiation, m_iSunsetRunVanGorsel, iRespirationColumn, iRadiationFilledColumn);
			}

			while(iReplaced != 0){
				if(m_bBootstraping == FALSE){
					if(aryMeanTermRE[i]->DetermineRespirationCurve(dTmax, dTmin, dTave, bUseNegativeValues, m_bUseLT_Equation, m_dMaximumQ10, m_dReferenceT_RE, m_dNighttimeRadiation, m_dUstar_threshold, iTemperatureColumn, iRadiationColumn, iRespirationColumn, iUstarColumn, bUseFixedData, m_aryFixedRespirationParameters[i], bAnnual) == FALSE){
						return;
					}
				}
				else{
					if(aryMeanTermRE[i]->DetermineRespirationCurve_bootstrap(dTmax, dTmin, dTave, bUseNegativeValues, m_bUseLT_Equation, m_dMaximumQ10, m_dReferenceT_RE, m_dNighttimeRadiation, m_dUstar_threshold, iTemperatureColumn, iRadiationColumn, iRespirationColumn, iUstarColumn, bUseFixedData, m_aryFixedRespirationParameters[i], bAnnual, m_iBootstrapNum) == FALSE){
						return;
					}
				}
					
				if(m_dQCThresholdSigma == 0.0 || iLoopCount == m_iQCMaximumLoop) break;
				if(iLoopCount != 0 && oldPrm.m_iDataNum / 2 > aryMeanTermRE[i]->m_RespirationParameters.m_iDataNum) break;

				if(iLoopCount == 0){
					oldPrm = aryMeanTermRE[i]->m_RespirationParameters;
				}
				iReplaced = aryMeanTermRE[i]->QC_RespirationData(m_dQCThresholdSigma, bUseNegativeValues, m_bUseLT_Equation, m_dNighttimeRadiation,  m_dUstar_threshold, iTemperatureColumn, iRadiationColumn,  iRespirationColumn, iUstarColumn);
				iLoopCount++;
			}

			if(iLoopCount != 0 && oldPrm.m_dQ10 != aryMeanTermRE[i]->s_dErrorValue && aryMeanTermRE[i]->m_RespirationParameters.m_dQ10 == aryMeanTermRE[i]->s_dErrorValue){
				aryMeanTermRE[i]->m_RespirationParameters = oldPrm;
			}
			iReplaced = -1;
			iLoopCount = 0;

			if(m_bBootstraping == FALSE){
				// f[^̏o
				strLine = aryMeanTermRE[i]->GetOutputLine_RespirationCurve(dTmax, dTmin, dTave);
				file.WriteString(strLine);

				ctlProgress.SetPos(5000 + dProgressSize * i);
			}
			else{
				// bootstrap p^̕ρAW΍vZ
				CRespirationParameters cResp = aryMeanTermRE[i]->Average_stdev_bootstrap_parameters_respiration();

				// Bootstraping p^̎擾
				m_aryRespirationParametersBootstrap.clear();
				aryMeanTermRE[i]->SetBootstrapRespirationParameters(m_aryRespirationParametersBootstrap, cResp, m_dMaximumQ10);

				// f[^̏o
				strLine = aryMeanTermRE[i]->GetOutputLine_RespirationCurve_bootstrap(cResp);
				file.WriteString(strLine);

				ctlProgress.SetPos(5000 / 2 + dProgressSize * i / 2);
			}
				
			// mۂ̈̊J
			if(m_bBootstraping == FALSE){
				aryMeanTermRE[i]->ReleaseDataset();
				delete aryMeanTermRE[i];
			}
		}

		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
	}

	if(m_bBootstraping == FALSE) aryMeanTermRE.clear();
}

BOOL CFluxPartitioningDlg::ReadFile_with_MovingWindow(CString& strInputFile, std::vector<CMeanTerm*>& aryMeanTermGPP, std::vector<CMeanTerm*>& aryMeanTermRE, int iRunPerDay, int iPhotosynthesisTerm, int iRespirationTerm, CString& strHeaderLine)
{
	//  ̓t@Cǂݍ݁Aw肳ꂽԖɐ؂蕪
	CString strLine;
	int iCountOfRun = 0;
	int iTermNum = 0;
	int iPeriod = 0;
	CString strTemp;
	BOOL bIsOK = FALSE;
	vector <CString> aryLines;

	try{
		CStdioFile InFile(strInputFile, CFile::modeRead | CFile::shareDenyWrite);

		// wb_[ǂݔ΂
		InFile.ReadString(strHeaderLine);	//	 1s
		InFile.ReadString(strLine);			//   2s

		while(InFile.ReadString(strLine)){
			if(strLine == ","){
				continue;
			}
			aryLines.push_back(strLine);
		}

		InFile.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return bIsOK;
	}

	// EBhEړăf[^i[@ipj
	if(m_bPhotosynthesis == TRUE){
		for(int i = 0; i < aryLines.size(); i++){
			if(i % iRunPerDay == 0 ){
				CMeanTerm* PhotosynthesisTerm = new CMeanTerm(iTermNum);
				aryMeanTermGPP.push_back(PhotosynthesisTerm);
				iTermNum++;

				// ̏擾
				if(m_bUsePeriod == TRUE){
					strTemp = aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1);
					iPeriod = atoi(strTemp);
					(aryMeanTermGPP.back())->SetCurrentPeriod(iPeriod);
				}
				strTemp = aryLines[i].Left(aryLines[i].Find(','));
				(aryMeanTermGPP.back())->SetCurrentDate(strTemp);

				int iStart = i - (iPhotosynthesisTerm - 1) / 2 * m_iRunPerDay;
				int iEnd   = i + (iPhotosynthesisTerm + 1) / 2 * m_iRunPerDay; // changed from -1 to +1 at 2010.01.27
				if(iStart < 0) iStart = 0;
				if(iEnd > (aryLines.size() - 1)) iEnd = aryLines.size() - 1;

				for(int j = iStart; j <= iEnd; j++){
					iPeriod = atoi(aryLines[j].Mid(aryLines[j].ReverseFind(',') + 1));
					if(m_bUsePeriod){
						(aryMeanTermGPP.back())->SetLine(aryLines[j], iPeriod);
					}
					else{
						(aryMeanTermGPP.back())->SetLine(aryLines[j]);
					}
				}
			}
		}
	}
	iTermNum = 0;

	// EBhEړăf[^i[@iċzpj
	if(m_bRespiration == TRUE){
		for(int i = 0; i < aryLines.size(); i++){
			if(i % iRunPerDay == 0 ){
				CMeanTerm* RespirationTerm = new CMeanTerm(iTermNum);
				aryMeanTermRE.push_back(RespirationTerm);
				iTermNum++;

				// ̏擾
				if(m_bUsePeriod == TRUE){
					strTemp = aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1);
					iPeriod = atoi(strTemp);
					(aryMeanTermRE.back())->SetCurrentPeriod(iPeriod);
				}
				strTemp = aryLines[i].Left(aryLines[i].Find(','));
				(aryMeanTermRE.back())->SetCurrentDate(strTemp);

				int iStart = i - (iRespirationTerm - 1) / 2 * m_iRunPerDay;
				int iEnd   = i + (iRespirationTerm + 1) / 2 * m_iRunPerDay; // changed from -1 to +1 at 2010.01.27
				if(iStart < 0) iStart = 0;
				if(iEnd > (aryLines.size() - 1)) iEnd = aryLines.size() - 1;

				for(int j = iStart; j <= iEnd; j++){
					iPeriod = atoi(aryLines[j].Mid(aryLines[j].ReverseFind(',') + 1));
					if(m_bUsePeriod){
						(aryMeanTermRE.back())->SetLine(aryLines[j], iPeriod);
					}
					else{
						(aryMeanTermRE.back())->SetLine(aryLines[j]);
					}
				}
			}
		}
		// ċzɂẮAʔNł1̃p^ZoB
		CMeanTerm* RespirationTerm = new CMeanTerm(iTermNum);
		aryMeanTermRE.push_back(RespirationTerm);
		(aryMeanTermRE.back())->SetCurrentPeriod(0);
		(aryMeanTermRE.back())->SetCurrentDate("Annual");
		for(i = 0; i < aryLines.size(); i++){
			iPeriod = atoi(aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1));
			(aryMeanTermRE.back())->SetLine(aryLines[i], 0);
		}
	}

	bIsOK = TRUE;
	return bIsOK;
}

BOOL CFluxPartitioningDlg::ReadFile_with_BlockWindow(CString& strInputFile, std::vector<CMeanTerm*>& aryMeanTermGPP, std::vector<CMeanTerm*>& aryMeanTermRE, int iRunPerDay, int iPhotosynthesisTerm, int iRespirationTerm, CString& strHeaderLine)
{
	//  ̓t@Cǂݍ݁Aw肳ꂽԖɐ؂蕪
	CString strLine;
	int iCountOfRun = 0;
	int iTermNum = 0;
	int iPeriod = 0;
	CString strTemp;
	BOOL bIsOK = FALSE;
	vector <CString> aryLines;

	try{
		CStdioFile InFile(strInputFile, CFile::modeRead | CFile::shareDenyWrite);

		// wb_[ǂݔ΂
		InFile.ReadString(strHeaderLine);	//	 1s
		InFile.ReadString(strLine);			//   2s

		while(InFile.ReadString(strLine)){
			if(strLine == ","){
				continue;
			}
			aryLines.push_back(strLine);
		}

		InFile.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return bIsOK;
	}

	// EBhEړăf[^i[@ipj
	BOOL bPeriodChange = FALSE;
	if(m_bPhotosynthesis == TRUE){
		for(int i = 0; i < aryLines.size(); i++){
			if(i % (iRunPerDay * iPhotosynthesisTerm) == 0 || bPeriodChange == TRUE){
				CMeanTerm* PhotosynthesisTerm = new CMeanTerm(iTermNum);
				aryMeanTermGPP.push_back(PhotosynthesisTerm);
				iTermNum++;
				bPeriodChange = FALSE;

				// ̏擾
				if(m_bUsePeriod == TRUE){
					strTemp = aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1);
					iPeriod = atoi(strTemp);
					(aryMeanTermGPP.back())->SetCurrentPeriod(iPeriod);
				}
				strTemp = aryLines[i].Left(aryLines[i].Find(','));
				(aryMeanTermGPP.back())->SetCurrentDate(strTemp);

				int iStart = i;
				int iEnd   = i + (iPhotosynthesisTerm * m_iRunPerDay);
				if(iEnd > (aryLines.size() - 1)) iEnd = aryLines.size() - 1;

				for(int j = iStart; j < iEnd; j++){
					// iPeriod = atoi(aryLines[j].Mid(aryLines[j].ReverseFind(',') + 1)); commented out at 2010.01.27

					// ǂݍݓr period ςƂ
					if(m_bUsePeriod && j != iStart && iPeriod != (aryMeanTermGPP.back())->GetCurrentPeriod()){
						bPeriodChange = TRUE;
						i = j - 1;
						break;
					}

					if(m_bUsePeriod){
						(aryMeanTermGPP.back())->SetLine(aryLines[j], iPeriod);
					}
					else{
						(aryMeanTermGPP.back())->SetLine(aryLines[j]);
					}
				}
			}
		}
	}
	iTermNum = 0;

	// EBhEړăf[^i[@iċzpj
	if(m_bRespiration == TRUE){
		for(int i = 0; i < aryLines.size(); i++){
			if(i % (iRunPerDay * iRespirationTerm) == 0 || bPeriodChange == TRUE){
				CMeanTerm* RespirationTerm = new CMeanTerm(iTermNum);
				aryMeanTermRE.push_back(RespirationTerm);
				iTermNum++;
				bPeriodChange = FALSE;

				// ̏擾
				if(m_bUsePeriod == TRUE){
					strTemp = aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1);
					iPeriod = atoi(strTemp);
					(aryMeanTermRE.back())->SetCurrentPeriod(iPeriod);
				}
				strTemp = aryLines[i].Left(aryLines[i].Find(','));
				(aryMeanTermRE.back())->SetCurrentDate(strTemp);

				int iStart = i;
				int iEnd   = i + (iRespirationTerm * m_iRunPerDay);
				if(iEnd > (aryLines.size() - 1)) iEnd = aryLines.size() - 1;

				for(int j = iStart; j < iEnd; j++){
					iPeriod = atoi(aryLines[j].Mid(aryLines[j].ReverseFind(',') + 1));

					// ǂݍݓr period ςƂ
					if(m_bUsePeriod && j != iStart && iPeriod != (aryMeanTermRE.back())->GetCurrentPeriod()){
						bPeriodChange = TRUE;
						i = j - 1;
						break;
					}

					if(m_bUsePeriod){
						(aryMeanTermRE.back())->SetLine(aryLines[j], iPeriod);
					}
					else{
						(aryMeanTermRE.back())->SetLine(aryLines[j]);
					}
				}
			}
		}
		// ċzɂẮAʔNł1̃p^ZoB
		CMeanTerm* RespirationTerm = new CMeanTerm(iTermNum);
		aryMeanTermRE.push_back(RespirationTerm);
		(aryMeanTermRE.back())->SetCurrentPeriod(0);
		(aryMeanTermRE.back())->SetCurrentDate("Annual");
		for(i = 0; i < aryLines.size(); i++){
			iPeriod = atoi(aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1));
			(aryMeanTermRE.back())->SetLine(aryLines[i], 0);
		}
	}

	bIsOK = TRUE;
	return bIsOK;
}

BOOL CFluxPartitioningDlg::ReadFile_with_MovingWindow(CString& strInputFile, std::vector<CMeanTerm*>& aryMeanTerm, int iRunPerDay, int iPhotosynthesisTerm, CString& strHeaderLine)
{
	//  ̓t@Cǂݍ݁Aw肳ꂽԖɐ؂蕪
	CString strLine;
	int iCountOfRun = 0;
	int iTermNum = 0;
	int iPeriod = 0;
	CString strTemp;
	BOOL bIsOK = FALSE;
	vector <CString> aryLines;

	try{
		CStdioFile InFile(strInputFile, CFile::modeRead | CFile::shareDenyWrite);

		// wb_[ǂݔ΂
		InFile.ReadString(strHeaderLine);	//	 1s
		InFile.ReadString(strLine);			//   2s

		while(InFile.ReadString(strLine)){
			if(strLine == ","){
				continue;
			}
			aryLines.push_back(strLine);
		}

		InFile.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return bIsOK;
	}

	// EBhEړăf[^i[@ipj
	for(int i = 0; i < aryLines.size(); i++){
		if(i % iRunPerDay == 0){
			CMeanTerm* PhotosynthesisTerm = new CMeanTerm(iTermNum);
			aryMeanTerm.push_back(PhotosynthesisTerm);
			iTermNum++;

			// ̏擾
			if(m_bUsePeriod == TRUE){
				strTemp = aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1);
				iPeriod = atoi(strTemp);
				(aryMeanTerm.back())->SetCurrentPeriod(iPeriod);
			}
			strTemp = aryLines[i].Left(aryLines[i].Find(','));
			(aryMeanTerm.back())->SetCurrentDate(strTemp);

			int iStart = i - (iPhotosynthesisTerm - 1) / 2 * m_iRunPerDay;
			int iEnd   = i + (iPhotosynthesisTerm + 1) / 2 * m_iRunPerDay; // changed from -1 to +1 at 2010.01.27
			if(iStart < 0) iStart = 0;
			if(iEnd > (aryLines.size() - 1)) iEnd = aryLines.size() - 1;

			for(int j = iStart; j <= iEnd; j++){
				iPeriod = atoi(aryLines[j].Mid(aryLines[j].ReverseFind(',') + 1));  // commented out at 2010.01.27  at 2011.01.01

				// ǂݍݓr period ςƂ 
				// added at 2011.2.1 for moving window 
				if(m_bUsePeriod && iPeriod != (aryMeanTerm.back())->GetCurrentPeriod()){
					continue;
				}

				if(m_bUsePeriod){
					(aryMeanTerm.back())->SetLine(aryLines[j], iPeriod);
				}
				else{
					(aryMeanTerm.back())->SetLine(aryLines[j]);
				}
			}
		}
	}

	bIsOK = TRUE;
	return bIsOK;
}

BOOL CFluxPartitioningDlg::ReadFile_with_BlockWindow(CString& strInputFile, std::vector<CMeanTerm*>& aryMeanTerm, int iRunPerDay, int iPhotosynthesisTerm, CString& strHeaderLine)
{
	//  ̓t@Cǂݍ݁Aw肳ꂽԖɐ؂蕪
	CString strLine;
	int iCountOfRun = 0;
	int iTermNum = 0;
	int iPeriod = 0;
	CString strTemp;
	BOOL bIsOK = FALSE;
	vector <CString> aryLines;

	try{
		CStdioFile InFile(strInputFile, CFile::modeRead | CFile::shareDenyWrite);

		// wb_[ǂݔ΂
		InFile.ReadString(strHeaderLine);	//	 1s
		InFile.ReadString(strLine);			//   2s

		while(InFile.ReadString(strLine)){
			if(strLine == ","){
				continue;
			}
			aryLines.push_back(strLine);
		}

		InFile.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return bIsOK;
	}

	// EBhEړăf[^i[@ipj
	BOOL bPeriodChange = FALSE;
	for(int i = 0; i < aryLines.size(); i++){
		if(i % (iRunPerDay * iPhotosynthesisTerm) == 0 || bPeriodChange == TRUE){
			CMeanTerm* PhotosynthesisTerm = new CMeanTerm(iTermNum);
			aryMeanTerm.push_back(PhotosynthesisTerm);
			iTermNum++;
			bPeriodChange = FALSE;

			// ̏擾
			if(m_bUsePeriod == TRUE){
				strTemp = aryLines[i].Mid(aryLines[i].ReverseFind(',') + 1);
				iPeriod = atoi(strTemp);
				(aryMeanTerm.back())->SetCurrentPeriod(iPeriod);
			}
			strTemp = aryLines[i].Left(aryLines[i].Find(','));
			(aryMeanTerm.back())->SetCurrentDate(strTemp);

			int iStart = i;
			int iEnd   = i + (iPhotosynthesisTerm * m_iRunPerDay);
			if(iEnd > (aryLines.size() - 1)) iEnd = aryLines.size() - 1;

			for(int j = iStart; j < iEnd; j++){
				// iPeriod = atoi(aryLines[j].Mid(aryLines[j].ReverseFind(',') + 1)); commented out at 2010.01.27

				// ǂݍݓr period ςƂ
				if(m_bUsePeriod && j != iStart && iPeriod != (aryMeanTerm.back())->GetCurrentPeriod()){
					bPeriodChange = TRUE;
					i = j - 1;
					break;
				}

				if(m_bUsePeriod){
					(aryMeanTerm.back())->SetLine(aryLines[j], iPeriod);
				}
				else{
					(aryMeanTerm.back())->SetLine(aryLines[j]);
				}
			}
		}
	}

	bIsOK = TRUE;
	return bIsOK;
}

BOOL CFluxPartitioningDlg::ReadFixedParamterFile()
{
		m_aryFixedPhotosynthesisParameters.clear();
		m_aryFixedRespirationParameters.clear();
		CString strLine, strTemp;
		double dTref = 10.0;
		BOOL bNew = FALSE; // 2012.02.27 for Tref input

		try{
			CStdioFile file(m_strFixedDataFile, CFile::modeRead | CFile::shareDenyWrite);

			int iCount = 0;
			while(file.ReadString(strLine)){
				iCount++;

				// sڂ̓wb_[Fǂݔ΂
				if(iCount == 1){
					// New Input Parameter for Tref 2012.02.27
					int iComma = strLine.Replace(',', '@');

					if(iComma == 11){
						bNew = TRUE;
					}

					if(iComma != 9 && iComma != 11){
						AfxMessageBox("The format of the fixed paramter file was inappropriate.");
						return FALSE;
					}
					continue;	
				}

				int iColmunNumber = 0;
				while(strLine.Find(",") != -1){
					iColmunNumber++;	// JJEg

					CPhotosynthesisParameters cPhotosynthesisParameters;
					CRespirationParameters cRespirationParameters;

					strLine = strLine.Mid(strLine.Find(',') + 1);	// date

					strTemp = strLine.Left(strLine.Find(','));
					cPhotosynthesisParameters.m_dMaximumPhotosynthesis = atof(strTemp);

					strLine = strLine.Mid(strLine.Find(',') + 1);
					strTemp = strLine.Left(strLine.Find(','));
					cPhotosynthesisParameters.m_dB = atof(strTemp);

					strLine = strLine.Mid(strLine.Find(',') + 1);
					strTemp = strLine.Left(strLine.Find(','));
					cPhotosynthesisParameters.m_dR0 = atof(strTemp);


					strLine = strLine.Mid(strLine.Find(',') + 1);
					strTemp = strLine.Left(strLine.Find(','));
					cRespirationParameters.m_dR0 = atof(strTemp);

					strLine = strLine.Mid(strLine.Find(',') + 1);
					strTemp = strLine.Left(strLine.Find(','));
					cRespirationParameters.m_dQ10 = atof(strTemp);

					if(bNew){
						strLine = strLine.Mid(strLine.Find(',') + 1);
						strTemp = strLine.Left(strLine.Find(','));
						cRespirationParameters.m_dTref = atof(strTemp);
					}

					strLine = strLine.Mid(strLine.Find(',') + 1);
					strTemp = strLine.Left(strLine.Find(','));
					cPhotosynthesisParameters.m_iFixed_Pmax_B = atoi(strTemp);

					strLine = strLine.Mid(strLine.Find(',') + 1);
					strTemp = strLine.Left(strLine.Find(','));
					cPhotosynthesisParameters.m_iFixed_R0 = atoi(strTemp);

					strLine = strLine.Mid(strLine.Find(',') + 1);
					cRespirationParameters.m_iFixed_Rref = atoi(strLine);

					strLine = strLine.Mid(strLine.Find(',') + 1);
					cRespirationParameters.m_iFixed_Q10 = atoi(strLine);

					if(bNew){
						strLine = strLine.Mid(strLine.Find(',') + 1);
						cRespirationParameters.m_iFixed_Tref = atoi(strLine);
					}

					m_aryFixedPhotosynthesisParameters.push_back(cPhotosynthesisParameters);
					m_aryFixedRespirationParameters.push_back(cRespirationParameters);

				}
			}

			file.Close();
		}
		catch(CException* e){
			e->ReportError();
			e->Delete();
			return FALSE;
		}
		return TRUE;
}

void CFluxPartitioningDlg::OnBtnApply() 
{
	BOOL bIsDebug = FALSE;
	#ifdef _DEBUG
	bIsDebug = TRUE;
	#endif

	CString strErrorMessage;
	if(bIsDebug){
		if(Gap_Filling_Carbon_Energy_Fluxes(strErrorMessage) == TRUE){
			AfxMessageBox("The calculation has been successfully done !");
		}
		else{
			AfxMessageBox(strErrorMessage);
		}
	}
	else{
		// Using Malti Thread
		if(!m_bIsThread){
			m_pThread = AfxBeginThread(OnApplyThread,(LPVOID)this);
			m_pThread->m_bAutoDelete = FALSE;
			m_pThread->ResumeThread();
		}
		else{
			AfxMessageBox("Error : Thread is busy !");
		}
	}
}

UINT CFluxPartitioningDlg::OnApplyThread(LPVOID pParam)
{
	CFluxPartitioningDlg* pDlg = static_cast<CFluxPartitioningDlg*>(pParam);

 	pDlg->m_bIsThread = TRUE;
	if(pDlg->Gap_Filling_Carbon_Energy_Fluxes(pDlg->m_strErrorMessageForThread) == TRUE){
		AfxMessageBox("The calculation has been successfully done !");
	}
	else{
		if(pDlg->m_strErrorMessageForThread != ""){
			AfxMessageBox(pDlg->m_strErrorMessageForThread);
		}
	}

	pDlg->m_bIsThread = FALSE;
	return 0;
}

BOOL CFluxPartitioningDlg::Gap_Filling_Carbon_Energy_Fluxes(CString& strErrorMessage) 
{
	UpdateData(TRUE);
	BeginWaitCursor();

	// **************************************************
	// vOXo[̏
	// **************************************************
	int iProgressRange = 600;
	m_ctlProgress.SetRange(0, iProgressRange);	// Inisilize Progress bar
	m_ctlProgress.SetPos(0);

	///////////////////////////////////////////
	// eLXg{bNX̓͂̃`FbN
	///////////////////////////////////////////
	if(ErrorCheckApply(strErrorMessage) == FALSE){
		return FALSE;
	}
	m_ctlProgress.SetPos(10);

	// ***************************************************************
	// ݒt@C̏o
	// ***************************************************************
	CString strOutSettingFile = m_strInputFileApply;
	strOutSettingFile = strOutSettingFile.Left(strOutSettingFile.ReverseFind('.')) + ".apply.set.txt";
	strOutSettingFile = m_strOutputFileApply + "\\" + strOutSettingFile.Mid(strOutSettingFile.ReverseFind('\\') + 1);
	SaveSetting(strOutSettingFile);

	///******************************************************************************
	// ϓω@@Ɓ@\@ɂ⊮ 
	///******************************************************************************
	CString strHeader;
	if(m_bMeanDiurnalVariation == TRUE){
		vector<CMeanTerm*> aryMeanTermGap;

		if(m_bMovingAverage){
			if(ReadFile_with_MovingWindow(m_strInputFileApply, aryMeanTermGap, m_iRunPerDay, m_iGapFillingTerm, strHeader) == FALSE){
				// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
				strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
				return FALSE;
			}
		}
		else{
			if(ReadFile_with_BlockWindow(m_strInputFileApply, aryMeanTermGap, m_iRunPerDay, m_iGapFillingTerm, strHeader) == FALSE){
				// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
				strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
				return FALSE;
			}
		}

		if(GapFill_mdv_lt(strHeader, aryMeanTermGap, m_bMovingAverage, m_ctlProgress) == FALSE){
			// mۂ̈̊J
			for(int i = 0; i < aryMeanTermGap.size(); i++){
				aryMeanTermGap[i]->ReleaseDataset();
				delete aryMeanTermGap[i];
			}
			return FALSE;
		}
	}
	m_ctlProgress.SetPos(200);

	///******************************************************************************
	// Non-Linear Regression method start 
	///******************************************************************************
	if(m_bNonLinearRegression == FALSE){
		// AfxMessageBox("The calculation has been successfully done!");
		m_ctlProgress.SetPos(iProgressRange);
		return TRUE;
	}

	///////////////////////////////////////////
	// ̓p^t@C̓ǂݍ
	///////////////////////////////////////////	
	vector<CMeanTerm*> aryMeanTerm;
	if(Read_GPP_Parameter(aryMeanTerm) == FALSE){
		// AfxMessageBox("Fail in reading the GPP parameter file.");
		strErrorMessage = "Fail in reading the GPP parameter file.";
		return FALSE;
	}
	if(Read_RE_Parameter(aryMeanTerm) == FALSE){
		// AfxMessageBox("Fail in reading the RE parameter file.");
		strErrorMessage = "Fail in reading the RE parameter file.";
		return FALSE;
	}

	///////////////////////////////////////////
	// ͘At@C̓ǂݍ
	///////////////////////////////////////////	
	int iHeaderNum = 2;
	if(ReadContinuousFile_for_EachTerm(m_strInputFileApply, aryMeanTerm, strHeader, iHeaderNum) == FALSE){
		// AfxMessageBox("Fail in reading the continuous data file.");
		strErrorMessage = "Fail in reading the continuous data file.";
		return FALSE;
	}

	///////////////////////////////////////////
	// ʃt@C̏o
	///////////////////////////////////////////
	CString strLine;
	CString strOutPass;
	CString strInputTitle = m_strInputFileApply.Mid(m_strInputFileApply.ReverseFind('\\') + 1);
	strInputTitle = strInputTitle.Mid(0, strInputTitle.ReverseFind('.'));
	if(m_bUseVanGorsel) strInputTitle += ".vG";

	int iUstar = 0;
	int iNEEraw = 3;
	int iRawRadiationColumn = 4;
	int iRadiationColumn = 7;
	int iTemperatureColumn = 8;
	int iVPDColumn = 9;
	int iPeriodColumn = 10;
	int iNEE_model_nlr1 = -1;
	int iRE = -1;
	int iNEE_nlr1 = -1;
	int iNEE_nlr2 = -1;
	int iNEE_model = -1;
	int iGPP = -1;
	int iGPP2 = -1;
	int iRE2 = -1;

	try{
		// ⊮̎nt@CSĂ̊Ԃɂ킽ďo͂邽߂̃t@C̃CX^X
		if(m_bMovingAverage)
			strOutPass.Format("%s\\%s_mw%d_gap_filled_carbon_fluxes.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
		else
			strOutPass.Format("%s\\%s_bw%d_gap_filled_carbon_fluxes.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
		// strOutPass = m_strOutputFileApply + "\\" + strInputTitle + "_gap_filled_carbon_fluxes.csv";	

		CStdioFile file(strOutPass, CFile::modeCreate | CFile::modeWrite);

		// All result & flag file ̃wb_Lq
		strLine = "Date, Radiation, Temperature, NEE_raw, NEE_nlr1, NEE_nlr2, NEE_model, GPP, RE, GPP2, RE2, Period\n";
		file.WriteString(strLine);
		strLine = "--, W m-2, degreeC, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1\n";
		file.WriteString(strLine);

		// t@CEtH_쐬Aʂۑ
		vector<int> aryOutColumnNum;
		double dSizeProgress = 50.0 / aryMeanTerm.size();

		for(int i = 0; i < aryMeanTerm.size(); i++){
			// poȐxvZ
			if((iNEE_model_nlr1 = aryMeanTerm[i]->ApplyLightPhotosynthesisCurve(strHeader, iRadiationColumn)) <= 0 ){
				// AfxMessageBox("Fail in calcularing NEE by non-linear regression1.");
				strErrorMessage = "Fail in calcularing NEE by non-linear regression1.";
				return FALSE;
			}		
			iNEE_model_nlr1 -= 1;

			// Ԍnċzʂ̌vZ (RE)
			if((iRE = aryMeanTerm[i]->ApplyRespirationCurve(iTemperatureColumn)) <= 0 ){
				// AfxMessageBox("Fail in calcularing respiration.");
				strErrorMessage = "Fail in calcularing respiration.";
				return FALSE;
			}
			iRE -= 1;

			// `NEE ⊮@i[Ȑ݂̂gpꍇj
			if((iNEE_nlr1 = aryMeanTerm[i]->GapFill_NEE_NonLinearRegression(iNEEraw, iUstar, iRadiationColumn, iNEE_model_nlr1)) <= 0 ){
				// AfxMessageBox("Fail in gap-filling NEE by using NEE by non-linear regression1.");
				strErrorMessage = "Fail in gap-filling NEE by using NEE by non-linear regression1.";
				return FALSE;
			}
			iNEE_nlr1 -= 1;

			// `NEE ⊮@i[Ȑƌċzgpꍇj
			if((iNEE_nlr2 = aryMeanTerm[i]->GapFill_NEE_NonLinearRegression(iNEEraw, iUstar, iRadiationColumn,  iNEE_model_nlr1, iRE)) <= 0 ){
				// AfxMessageBox("Fail in gap-filling NEE by using NEE by non-linear regression2.");
				strErrorMessage = "Fail in gap-filling NEE by using NEE by non-linear regression2.";
				return FALSE;
			}
			iNEE_nlr2 -= 1;

			// `NEE 𐄒@i[ȐƌċzɂNEE ̐j
			if((iNEE_model = aryMeanTerm[i]->Estimate_NEE_NonLinearRegression(iRadiationColumn,  iNEE_model_nlr1, iRE)) <= 0 ){
				// AfxMessageBox("Fail in estimating NEE by using NEE by non-linear regression1.");
				strErrorMessage = "Fail in estimating NEE by using NEE by non-linear regression1.";
				return FALSE;
			}
			iNEE_model -= 1;

			// GPP @iGPP ̐j
			if((iGPP = aryMeanTerm[i]->Estimate_GPP(iNEE_nlr2,  iRE)) <= 0 ){
				// AfxMessageBox("Fail in estimating GPP.");
				strErrorMessage = "Fail in estimating GPP.";
				return FALSE;
			}
			iGPP -= 1;

			if((iGPP2 = aryMeanTerm[i]->Adjust_GPP_RE(iGPP,  iRE, iGPP2, iRE2)) <= 0 ){
				// AfxMessageBox("Fail in estimating GPP.");
				strErrorMessage = "Fail in estimating GPP.""";
				return FALSE;
			}
			iGPP2 -= 1;
			iRE2 -= 1;

			// ʃt@C̏o
			aryOutColumnNum.push_back(iRawRadiationColumn);	// radiation raw
			aryOutColumnNum.push_back(iTemperatureColumn);	// temperature raw
			aryOutColumnNum.push_back(iNEEraw);				// NEE raw
			aryOutColumnNum.push_back(iNEE_nlr1);			// NEE none linear regeression 1
			aryOutColumnNum.push_back(iNEE_nlr2);			// NEE none linear regeression 2
			aryOutColumnNum.push_back(iNEE_model);			// NEE model
			aryOutColumnNum.push_back(iGPP);				// GPP
			aryOutColumnNum.push_back(iRE);					// RE
			aryOutColumnNum.push_back(iGPP2);				// GPP adjusted
			aryOutColumnNum.push_back(iRE2);				// RE adjusted
			aryOutColumnNum.push_back(iPeriodColumn);		// Period
			aryMeanTerm[i]->WriteGapFilledCarbonFlux(file, aryOutColumnNum);
			aryOutColumnNum.clear();

			// mۂ̈̊J
			aryMeanTerm[i]->ReleaseDataset();
			delete aryMeanTerm[i];

			m_ctlProgress.SetPos(200 + dSizeProgress * i);
		}
		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
	}

	aryMeanTerm.clear();
	m_ctlProgress.SetPos(250);

	///******************************************************************************
	// GPP p^̌
	///******************************************************************************
	vector<CMeanTerm*> aryMeanTermGPP;
	Determine_GPP_Parameters(strOutPass, aryMeanTermGPP, strHeader, m_ctlProgress);
	m_ctlProgress.SetPos(450);

	///******************************************************************************
	// `ɂ⊮ꂽf[^ɑ΂`⊮
	///******************************************************************************
	LinearRegression_NonLinearRegression(strOutPass, aryMeanTermGPP, strHeader);
	m_ctlProgress.SetPos(500);

	if(m_bMeanDiurnalVariation == FALSE){
		m_ctlProgress.SetPos(600);
		return TRUE;
	}

	///******************************************************************************
	// LUT, NLR, MDV @ɂ錇f[^̍ŏIIȕ⊮sȂ
	// D揇ʂ́A1. Linear Regression, 2. LUT, 3. NLR, 4.MDV
	///******************************************************************************
	if(Final_Priority(aryMeanTermGPP, strInputTitle) == FALSE){
		return FALSE;
	}
	m_ctlProgress.SetPos(600);


	// mۂ̈̊J
	(aryMeanTermGPP.back())->ReleaseDataset();	

	// AfxMessageBox("The calculation has been successfully done!");
	return TRUE;
}

BOOL CFluxPartitioningDlg::Final_Priority(std::vector<CMeanTerm*>& aryMeanTermGPP, const CString& strInputTitle)
{
	vector<CMeanTerm*> aryMeanTermMDV;
	CString strOutput_mdv_file;
	CString strHeader, strLine;
	vector<CString> aryLines;
	vector<double> aryFinalH;
	vector<double> aryFinallE;
	vector<double> aryFinalNEE;
	vector<double> aryFinalGPP;
	vector<double> aryFinalRE;
	vector<int> aryFlagH;
	vector<int> aryFlaglE;
	vector<int> aryFlagNEE;
	vector<CString> aryDate;

	// Look up table and Mean diurnal variation t@C쐬
	if(m_bMovingAverage){
		strOutput_mdv_file.Format("%s\\%s_mw%d_gap_filled_mdv_lt.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
	}
	else{
		strOutput_mdv_file.Format("%s\\%s_bw%d_gap_filled_mdv_lt.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
	}

	try{
		CStdioFile InFile(strOutput_mdv_file, CFile::modeRead | CFile::shareDenyWrite);

		// wb_[ǂݔ΂
		InFile.ReadString(strHeader);		//	 1s
		InFile.ReadString(strLine);			//   2s

		while(InFile.ReadString(strLine)){
			if(strLine == ","){
				continue;
			}
			aryLines.push_back(strLine);
		}

		InFile.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return FALSE;
	}

	CMeanTerm* MdvTerm = new CMeanTerm(aryLines.size());
	aryMeanTermMDV.push_back(MdvTerm);
	for(int i = 0; i < aryLines.size(); i++){
		(aryMeanTermMDV.back())->SetLine(aryLines[i]);
	}
	// ԂƂɕf[^AꍀڂƂ̃f[^Zbgɐ؂蕪	
	CString strTemp = strHeader;
	if((aryMeanTermMDV.back())->SetDataset(strTemp.Replace(',','@')) == FALSE){
		return FALSE;
	}
	(aryMeanTermMDV.back())->SetHeader(strHeader);

	// ̕⊮̗D揇ʂl
	(aryMeanTermMDV.back())->Set_Final_Gap_Filled_Data(*aryMeanTermGPP.back(), 
		aryDate, aryFinalH, aryFinallE, aryFinalNEE, aryFlagH, aryFlaglE, aryFlagNEE, aryFinalGPP, aryFinalRE);

	// f[^̏o
	try{
		CString strOutFinalFile;
		if(m_bMovingAverage){
			strOutFinalFile.Format("%s\\%s_mw%d_final.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
		}
		else{
			strOutFinalFile.Format("%s\\%s_bw%d_final.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
		}
		CStdioFile file(strOutFinalFile, CFile::modeCreate | CFile::modeWrite);

		// wb_[Lq
		strLine.Format("Date, H_gp, lE_gp, NEE_gp, GPP, RE, H_flag, lE_flag, NEE_flag\n");
		file.WriteString(strLine);
		strLine.Format("--, W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, --, --, --\n");
		file.WriteString(strLine);

		for(i = 0; i < aryFinalH.size(); i++){
			strLine.Format("%s, %lf,%lf,%lf,%lf,%lf,%d,%d,%d\n", aryDate[i], aryFinalH[i], aryFinallE[i], aryFinalNEE[i], aryFinalGPP[i], aryFinalRE[i], aryFlagH[i], aryFlaglE[i], aryFlagNEE[i]);
			file.WriteString(strLine);
		}
		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return FALSE;
	}


	// mۂ̈̊J
	(aryMeanTermMDV.back())->ReleaseDataset();

	return TRUE;
}

void CFluxPartitioningDlg::LinearRegression_NonLinearRegression(const CString& strOutPass, std::vector<CMeanTerm*>& aryMeanTerm, CString& strHeader)
{
	CString strLine;
	int iNEEraw = 2;
	int iNEE_nlr1 = 3;
	int iNEE_nlr2 = 4;
	int iNEE_model = 5;
	int iGPP = 6;
	int iRE = 7;
	int iGPP2 = 8;
	int iRE2 = 9;
	vector <CString> aryLines;

	try{
		CStdioFile InFile(strOutPass, CFile::modeRead | CFile::shareDenyWrite);

		// wb_[ǂݔ΂
		InFile.ReadString(strHeader);		//	 1s
		InFile.ReadString(strLine);			//   2s

		while(InFile.ReadString(strLine)){
			if(strLine == ","){
				continue;
			}
			aryLines.push_back(strLine);
		}

		InFile.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return;
	}

	CMeanTerm* PhotosynthesisTerm = new CMeanTerm(aryLines.size());
	aryMeanTerm.push_back(PhotosynthesisTerm);
	for(int i = 0; i < aryLines.size(); i++){
		(aryMeanTerm.back())->SetLine(aryLines[i]);
	}
	// ԂƂɕf[^AꍀڂƂ̃f[^Zbgɐ؂蕪	
	CString strTemp = strHeader;
	if((aryMeanTerm.back())->SetDataset(strTemp.Replace(',','@')) == FALSE){
		return;
	}
	(aryMeanTerm.back())->SetHeader(strHeader);

	
	// `⊮
	if(m_bUseLinearRegression){
		if((aryMeanTerm.back())->LinearInterpolation(m_iMaximumForLinearRegression, FALSE) == FALSE){
			return;
		}
		(aryMeanTerm.back())->LinearInterpolation_NoneLinearRegression(m_iMaximumForLinearRegression, iNEEraw, iNEE_nlr1, iNEE_nlr2);
	}
	try{
		CStdioFile file(strOutPass, CFile::modeCreate | CFile::modeWrite);

		// All result & flag file ̃wb_Lq
		strLine = "Date, NEE_raw, NEE_nlr1, NEE_nlr2, NEE_model, GPP, RE, GPP2, RE2\n";
		file.WriteString(strLine);
		strLine = "--, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1\n";
		file.WriteString(strLine);

		vector<int> aryOutColumnNum;
		aryOutColumnNum.push_back(iNEEraw);
		aryOutColumnNum.push_back(iNEE_nlr1);
		aryOutColumnNum.push_back(iNEE_nlr2);
		aryOutColumnNum.push_back(iNEE_model);
		aryOutColumnNum.push_back(iGPP);
		aryOutColumnNum.push_back(iRE);
		aryOutColumnNum.push_back(iGPP2);
		aryOutColumnNum.push_back(iRE2);
		(aryMeanTerm.back())->WriteGapFilledCarbonFlux(file, aryOutColumnNum);
		aryOutColumnNum.clear();

		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
	}
}
	
void CFluxPartitioningDlg::Determine_GPP_Parameters(CString& strOutPass, std::vector<CMeanTerm*>& aryMeanTermGPP, CString& strHeaderLine, CProgressCtrl& ctlProgress)
{
	BOOL bUseNegativeValues = FALSE;
	BOOL bCalculateR0 = FALSE;
	CPhotosynthesisParameters cDammy;
	int iHeaderNum = 1;
	int iRadiation = 0;
	int iTemperatureFilledColumn = 1;
	int iNEEraw = 2;
	int iNEE_nlr1 = 3;
	int iNEE_nlr2 = 4;
	int iGPP = 8;
	int iGPPobs = -1;

	CString strInputTitle = m_strInputFileApply.Mid(m_strInputFileApply.ReverseFind('\\') + 1);
	strInputTitle = strInputTitle.Mid(0, strInputTitle.ReverseFind('.'));
	if(m_bUseVanGorsel) strInputTitle += ".vG";
	
	if(m_bMovingAverage){
		if(ReadFile_with_MovingWindow(strOutPass, aryMeanTermGPP, m_iRunPerDay, m_iGapFillingTerm, strHeaderLine) == FALSE){
			AfxMessageBox("Fail in reading the continuous data file for the GPP parameter estimation.");
			return;
		}
	}
	else{
		if(ReadFile_with_BlockWindow(strOutPass, aryMeanTermGPP, m_iRunPerDay, m_iGapFillingTerm, strHeaderLine) == FALSE){
			AfxMessageBox("Fail in reading the continuous data file for the GPP parameter estimation.");
			return;
		}
	}

	///////////////////////////////////////////
	// t@CEtH_쐬Aʂۑ
	///////////////////////////////////////////	
	CString strOutputFileName;
	if(m_bUseNonRectangular == 0){
		if(m_bMovingAverage)
			strOutputFileName.Format("%s\\%s_mw%d_GPP-curve.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
		else
			strOutputFileName.Format("%s\\%s_bw%d_GPP-curve.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
	}
	else{
		if(m_bMovingAverage)
			strOutputFileName.Format("%s\\%s_mw%d_nGPP-curve_%.2lf.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm, m_dConvexity);
		else
			strOutputFileName.Format("%s\\%s_bw%d_nGPP-curve_%.2lf.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm, m_dConvexity);
	}

	try{
		CStdioFile file(strOutputFileName, CFile::modeCreate | CFile::modeWrite);

		// sڂɗLq
		double dTmax = -99999, dTmin = -99999, dTave = -99999;
		CString strLine;
		strLine.Format("Date, GPPmax, b, R0, RMSE, R, Tmax, Tmin, Tave, Data Number, iCount, Negative Values:%d, Non Rectangular:%d, Convexity:%lf\n", bUseNegativeValues, m_bUseNonRectangular, m_dConvexity);
		file.WriteString(strLine);

		double dProgressSize = 200.0 / aryMeanTermGPP.size();
		 for(int i = 0; i < aryMeanTermGPP.size(); i++){
			// ԂƂɕf[^AꍀڂƂ̃f[^Zbgɐ؂蕪	
			CString strTemp = strHeaderLine;
			if(aryMeanTermGPP[i]->SetDataset(strTemp.Replace(',','@')) == FALSE){
				return;
			}
			aryMeanTermGPP[i]->SetHeader(strHeaderLine);

			if((iGPPobs = aryMeanTermGPP[i]->SetObservedGPP(iNEEraw, iGPP)) < 0){
				return;
			}
			iGPPobs -= 1;

			// poȐ̌
			if(aryMeanTermGPP[i]->DetermineLightPhotosynthesisCurve(m_bSetMaximumB, m_dMaximumB, m_dConvexity, m_dNighttimeRadiation, bCalculateR0, bUseNegativeValues, m_bUseNonRectangular, iRadiation, iGPPobs, FALSE, cDammy) == FALSE){
				return;
			}

			// ōEŒEϋČ
			aryMeanTermGPP[i]->GetMaximumMinimuAverage(iTemperatureFilledColumn, dTmax, dTmin, dTave);

			// f[^̏o
			strLine = aryMeanTermGPP[i]->GetOutputLine_LightPhotosynthesis(bUseNegativeValues, dTmax, dTmin, dTave);
			file.WriteString(strLine);

			// mۂ̈̊J
			aryMeanTermGPP[i]->ReleaseDataset();
			delete aryMeanTermGPP[i];

			ctlProgress.SetPos(250 + dProgressSize * i);
		}

		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
	}
	aryMeanTermGPP.clear();
}

BOOL CFluxPartitioningDlg::ErrorCheckApply(CString& strErrorMessage)
{
	
	if(m_bNonLinearRegression == FALSE && m_bMeanDiurnalVariation == FALSE){
		return FALSE;
	}
	if(m_strInputFileApply == ""){
		// AfxMessageBox ("Please select the input file for application.");
		strErrorMessage = "Please select the input file for application.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_INPUT_APLLY));
		return FALSE;
	}
	if(m_strOutputFileApply == ""){
		// AfxMessageBox ("Please select the output file for application.");
		strErrorMessage = "Please select the output file for application.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_OUTPUT_APLLY));
		return FALSE;
	}
	if(m_bNonLinearRegression == TRUE && m_strInputGPP == ""){
		// AfxMessageBox ("Please select the input parameter file for photosynthesis.");
		strErrorMessage = "Please select the input parameter file for photosynthesis.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_INPUT_GPP));
		return FALSE;
	}
	if(m_bNonLinearRegression == TRUE && m_strInputRE == ""){
		// AfxMessageBox ("Please select the input parameter file for respiration.");
		strErrorMessage = "Please select the input parameter file for respiration.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_INPUT_RE));
		return FALSE;
	}
	if(m_bMovingAverage && m_bMeanDiurnalVariation == TRUE && m_iGapFillingTerm % 2 == 0){
		// AfxMessageBox ("The gap-filling term should be odd number.");
		strErrorMessage = "The gap-filling term should be odd number.";
		(this)->GotoDlgCtrl(GetDlgItem(IDC_EDIT_GAP_TERM));
		return FALSE;
	}

	return TRUE;
}

BOOL CFluxPartitioningDlg::Read_GPP_Parameter(std::vector<CMeanTerm*>& aryMeanTerm)
{
	try{
		CStdioFile file(m_strInputGPP, CFile::modeRead | CFile::shareDenyWrite);
		CString strLine, strTemp;
		BOOL bUseNegativeValues;
		BOOL bUseNonRectangular;
		double dDate;
		double dConvexity;
		double dAmax;
		double dB;
		double dR0;		

		// sڂɗLq
		int iCount = 0;

		while(file.ReadString(strLine)){			
			if(strLine == ","){
				continue;
			}
			iCount++; 

			// Get Header data
			if(iCount == 1){
				// Get bUseNegativeValues
				if(strLine.Find("Negative Values:") == -1){
					AfxMessageBox ("p^t@CtH[}bg܂B");
					return FALSE;
				}
				strTemp = strLine.Mid(strLine.Find("Negative Values:"));
				strTemp = strTemp.Mid(0, strTemp.Find(","));
				strTemp = strTemp.Mid(strTemp.Find(":") + 1);
				bUseNegativeValues = atoi(strTemp);

				// Get bUseNonRectangular
				if(strLine.Find("Non Rectangular:") == -1){
					AfxMessageBox ("p^t@CtH[}bg܂B");
					return FALSE;
				}
				strTemp = strLine.Mid(strLine.Find("Non Rectangular:"));
				strTemp = strTemp.Mid(0, strTemp.Find(","));
				strTemp = strTemp.Mid(strTemp.Find(":") + 1);
				bUseNonRectangular = atoi(strTemp);

				// Get dConvexity
				if(strLine.Find("Convexity:") == -1){
					AfxMessageBox ("p^t@CtH[}bg܂B");
					return FALSE;
				}
				strTemp = strLine.Mid(strLine.Find("Convexity:"));
				strTemp = strTemp.Mid(strTemp.Find(":") + 1);
				dConvexity = atof(strTemp);				

				continue;
			}

			// Header ȊÕf[^̓ǂݍ
			CMeanTerm* MeanTerm = new CMeanTerm(iCount - 2);
			if(iCount == 2) MeanTerm->SetPhotosynthesisConstant(bUseNegativeValues, bUseNonRectangular, dConvexity);
			// Get Date
			strTemp = strLine.Mid(0, strLine.Find(","));
			dDate = atof(strTemp);
			// Get dAmax
			strLine = strLine.Mid(strLine.Find(",") + 1);
			strTemp = strLine.Mid(0, strLine.Find(","));
			dAmax = atof(strTemp);
			// Get dB
			strLine = strLine.Mid(strLine.Find(",") + 1);
			strTemp = strLine.Mid(0, strLine.Find(","));
			dB = atof(strTemp);
			// Get dR0
			strLine = strLine.Mid(strLine.Find(",") + 1);
			strTemp = strLine.Mid(0, strLine.Find(","));
			dR0 = atof(strTemp);
			if(bUseNegativeValues == TRUE && dR0 != CDataset::s_dErrorValue) dR0 = - dR0;
			// Set Parameters
			MeanTerm->SetParametersGPP(dDate, dAmax, dB, dR0);
			aryMeanTerm.push_back(MeanTerm);
		}

		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return FALSE;
	}
	return TRUE;
}

BOOL CFluxPartitioningDlg::Read_RE_Parameter(std::vector<CMeanTerm*>& aryMeanTerm)
{
	double dR0_annual, dQ10_annual, dTref_annual;
	BOOL bNew = FALSE;

	try{
		CStdioFile file(m_strInputRE, CFile::modeRead | CFile::shareDenyWrite);
		CString strLine, strTemp;
		BOOL bUseNegativeValues;
		BOOL bUseLT_Equation;
		double dDate;
		double dR0, dQ10, dTref = 10.0;
		double dUstar_threshold;
		double dNighttimeRadiation;

		// sڂɗLq
		int iCount = 0;

		while(file.ReadString(strLine)){			
			if(strLine == ","){
				continue;
			}
			iCount++; 

			// Get Header data
			if(iCount == 1){
				// Get bUseNegativeValues
				if(strLine.Find("Negative Values:") == -1){
					AfxMessageBox ("p^t@CtH[}bg܂B");
					return FALSE;
				}
				strTemp = strLine.Mid(strLine.Find("Negative Values:"));
				strTemp = strTemp.Mid(0, strTemp.Find(","));
				strTemp = strTemp.Mid(strTemp.Find(":") + 1);
				bUseNegativeValues = atoi(strTemp);

				// Get bUseLT_Equation
				if(strLine.Find("Lloyd and Taylor:") == -1){
					AfxMessageBox ("p^t@CtH[}bg܂B");
					return FALSE;
				}
				strTemp = strLine.Mid(strLine.Find("Lloyd and Taylor:"));
				strTemp = strTemp.Mid(0, strTemp.Find(","));
				strTemp = strTemp.Mid(strTemp.Find(":") + 1);
				bUseLT_Equation = atoi(strTemp);

				// Get dUstar_threshold
				if(strLine.Find("u* threshold:") == -1){
					AfxMessageBox ("p^t@CtH[}bg܂B");
					return FALSE;
				}
				strTemp = strLine.Mid(strLine.Find("u* threshold:"));
				strTemp = strTemp.Mid(strTemp.Find(":") + 1);
				dUstar_threshold = atof(strTemp);	
				
				// Get dNighttimeRadiation
				if(strLine.Find("Nighttime radiation:") == -1){
					AfxMessageBox ("p^t@CtH[}bg܂B");
					return FALSE;
				}
				strTemp = strLine.Mid(strLine.Find("Nighttime radiation:"));
				strTemp = strTemp.Mid(strTemp.Find(":") + 1);
				dNighttimeRadiation = atof(strTemp);

				// New Input Parameter for Tref 2012.02.27
				if(strLine.Find("Tref,") != -1){
					bNew = TRUE;
				}

				continue;
			}			

			// Header ȊÕf[^̓ǂݍ
			if(iCount == 2){
				aryMeanTerm[iCount - 2]->SetRespirationConstant(bUseNegativeValues, bUseLT_Equation, dUstar_threshold, dNighttimeRadiation);
			}
			// Get Date
			strTemp = strLine.Mid(0, strLine.Find(","));
			dDate = atof(strTemp);
			// Get dR0
			strLine = strLine.Mid(strLine.Find(",") + 1);
			strTemp = strLine.Mid(0, strLine.Find(","));
			dR0 = atof(strTemp);
			// Get dQ10
			strLine = strLine.Mid(strLine.Find(",") + 1);
			strTemp = strLine.Mid(0, strLine.Find(","));
			dQ10 = atof(strTemp);
			// Get dTref
			if(bNew){
				strLine = strLine.Mid(strLine.Find(",") + 1);
				strTemp = strLine.Mid(0, strLine.Find(","));
				dTref = atof(strTemp);
			}

			if(iCount - 2 >= aryMeanTerm.size()){
				dR0_annual = dR0;
				dQ10_annual = dQ10;
				dTref_annual = dTref;
				continue;
			}

			// if(bUseNegativeValues == TRUE && dR0 != CDataset::s_dErrorValue) dR0 = - dR0;
			// Set Parameters
			aryMeanTerm[iCount - 2]->SetParametersRE(dDate, dR0, dQ10, dTref);
			// aryMeanTerm.push_back(MeanTerm);
		}

		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return FALSE;
	}

	// gap-fill for the error data by using annual data
	for(int i = 0; i < aryMeanTerm.size(); i++){
		aryMeanTerm[i]->UpdateParametersRE(dR0_annual, dQ10_annual, dTref_annual, CDataset::s_dErrorValue);
	}

	return TRUE;
}

BOOL CFluxPartitioningDlg::ReadContinuousFile_for_EachTerm(const CString strFileName, std::vector<CMeanTerm*>& aryMeanTerm, CString& strHeader, int iHeaderNum)
{
	int iCountOfRun = 0;
	CString strLine, strTemp;
	double dDate;
	try{
		CStdioFile file(strFileName, CFile::modeRead | CFile::shareDenyWrite);
		int iCount = 0;

		while(file.ReadString(strLine)){
			if(strLine == ",,,,,,,,,,,"){
				continue;
			}
			iCount++; 

			// wb_[ǂݔ΂
			if(iCount <= iHeaderNum){
				if(iCount == 1) strHeader = strLine;
				continue;
			}

			// Get Date
			strTemp = strLine.Mid(0, strLine.Find(","));
			dDate = atof(strTemp);

			if(iCountOfRun == aryMeanTerm.size() - 1){
				aryMeanTerm[iCountOfRun]->SetLine(strLine);		//	 Ō̍s̓ǂݍ
			}
			else{
				if(dDate < aryMeanTerm[iCountOfRun + 1]->GetStartDay()){
					aryMeanTerm[iCountOfRun]->SetLine(strLine);
				}
				else{
					iCountOfRun++;
					aryMeanTerm[iCountOfRun]->SetLine(strLine);
				}
			}
		}
		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return FALSE;
	}
	return TRUE;
}

BOOL CFluxPartitioningDlg::GapFill_mdv_lt(const CString& strHeaderLine, std::vector<CMeanTerm*>& aryMeanTerm, BOOL bMovingAverage, CProgressCtrl& ctlProgress)
{
	
	int iUstarColumn = 0;
	int iHeatFlux = 1;
	int iLatentFlux = 2;
	int iNEE = 3;
	int iRadiationColumn = 4;
	int iTemperatureColumn = 5;
	int iVPDColumn = 6;
	int iRadiationFilledColumn = 7;
	int iTemperatureFilledColumn = 8;
	int iVPDFilledColumn = 9;
	int iAverageNumber = 5;
	int iCorrectRadiation_MDV = FALSE;
	
	vector<int> aryOutColumnNum;
	aryOutColumnNum.push_back(iHeatFlux);			// sensible heat flux raw
	aryOutColumnNum.push_back(iLatentFlux);			// latent heat raw
	aryOutColumnNum.push_back(iNEE);				// NEE raw

	///////////////////////////////////////////
	// t@CEtH_쐬Aʂۑ
	///////////////////////////////////////////	
	CString strOutputFileName, strOutputStdevFileName;
	CString strInputTitle = m_strInputFileApply.Mid(m_strInputFileApply.ReverseFind('\\') + 1);
	strInputTitle = strInputTitle.Mid(0, strInputTitle.ReverseFind('.'));
	if(m_bUseVanGorsel) strInputTitle += ".vG";

	if(bMovingAverage){
		strOutputFileName.Format("%s\\%s_mw%d_gap_filled_mdv_lt.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
	}
	else{
		strOutputFileName.Format("%s\\%s_bw%d_gap_filled_mdv_lt.csv", m_strOutputFileApply, strInputTitle, m_iGapFillingTerm);
	}
	if(m_bBootstraping){
		CString strTempFileName = strOutputFileName.Mid(0, strOutputFileName.ReverseFind('.'));
		strOutputFileName.Format("%s_bootstrap.csv", strTempFileName);
	}

	try{
		CStdioFile file(strOutputFileName, CFile::modeCreate | CFile::modeWrite);

		// sڂɗLq
		CString strLine;
		if(m_bBootstraping == FALSE){
			strLine.Format("Date, H_raw, lE_raw, NEE_raw, H_mdv, lE_mdv, NEE_mdv, H_lt, lE_lt, NEE_lt, H_pattern, lE_pattern, NEE_parttern, H_table, lE_table, NEE_table, GPP_lt, RE_lt\n");
			file.WriteString(strLine);
			strLine.Format(", W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1\n");
			file.WriteString(strLine);
		}
		else{
			strLine.Format("Date, H_raw, lE_raw, NEE_raw, H_mdv, lE_mdv, NEE_mdv, H_lt, lE_lt, NEE_lt, H_pattern, lE_pattern, NEE_parttern, H_table, lE_table, NEE_table\n");
			file.WriteString(strLine);
			strLine.Format(", W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1\n");
			file.WriteString(strLine);
		}

		double dProgressSize = 190.0 / aryMeanTerm.size();
		for(int i = 0; i < aryMeanTerm.size(); i++){
			// ԂƂɕf[^AꍀڂƂ̃f[^Zbgɐ؂蕪	
			CString strTemp = strHeaderLine;
			if(aryMeanTerm[i]->SetDataset(strTemp.Replace(',','@')) == FALSE){
				return FALSE;
			}
			aryMeanTerm[i]->SetHeader(strHeaderLine);
			aryMeanTerm[i]->SetRunPerDay(m_iRunPerDay);

			// eliminate negative nighttime NEE
			if(m_bEliminateNegativeNEE){
				if(aryMeanTerm[i]->Eliminate_nighttime_negative(aryOutColumnNum, m_dNighttimeRadiation, iNEE, iRadiationFilledColumn) == FALSE){
					return FALSE;
				}
			}

			// u* filtering
			if(aryMeanTerm[i]->Ustar_filtering(aryOutColumnNum, m_dNighttimeRadiation, m_dUstar_threshold, iUstarColumn, iRadiationFilledColumn) == FALSE){
				return FALSE;
			}

			// van Gorsel filtering added 2014.06.24
			if(m_bUseVanGorsel){
				aryMeanTerm[i]->VanGorsel_filtering(m_dNighttimeRadiation, m_iSunsetRunVanGorsel, iNEE, iRadiationFilledColumn);
			}

			// ϓω@ɂ⊮ 
			if(aryMeanTerm[i]->MeanDiurnalVariation(m_iRunPerDay, iRadiationFilledColumn, iAverageNumber, bMovingAverage, iCorrectRadiation_MDV, m_bBootstraping, m_iBootstrapNum) == FALSE){
				return FALSE;
			}

			if(m_bBootstraping == FALSE){
				// bNAbve[uɂ⊮ 
				if(aryMeanTerm[i]->LookUpTable(iHeatFlux, iLatentFlux, iNEE, iRadiationColumn, iTemperatureColumn, iVPDColumn, 
					iRadiationFilledColumn, iTemperatureFilledColumn, iVPDFilledColumn, m_dNighttimeRadiation, m_iRunPerDay, bMovingAverage) == FALSE){
					return FALSE;
				}

				// bNAbve[uɂċz 
				if(aryMeanTerm[i]->LookUpTable_respiration(iNEE, iRadiationColumn, iTemperatureColumn,
					iRadiationFilledColumn, iTemperatureFilledColumn, m_dNighttimeRadiation, m_iRunPerDay, bMovingAverage) == -1){
					return FALSE;
				}
			}
			else{
				// bNAbve[uɂ⊮ 
				if(aryMeanTerm[i]->LookUpTable_bootstrap(iHeatFlux, iLatentFlux, iNEE, iRadiationColumn, iTemperatureColumn, iVPDColumn, 
					iRadiationFilledColumn, iTemperatureFilledColumn, iVPDFilledColumn, m_dNighttimeRadiation, m_iRunPerDay, bMovingAverage, m_iBootstrapNum) == FALSE){
					return FALSE;
				}
			}

			// `⊮
			if(m_bUseLinearRegression){
				if(aryMeanTerm[i]->LinearInterpolation(m_iMaximumForLinearRegression, bMovingAverage) == FALSE){
					return FALSE;
				}
			}

			// f[^̏o
			aryMeanTerm[i]->WriteGapFilledCarbonFlux_mdv_lt(file, aryOutColumnNum, bMovingAverage, m_bBootstraping);

			if(m_bBootstraping == FALSE){
				aryMeanTerm[i]->ReleaseDataset();
				delete aryMeanTerm[i];

				ctlProgress.SetPos(10 + i * dProgressSize);
			}
		}

		file.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return FALSE;
	}

	if(m_bBootstraping == FALSE) aryMeanTerm.clear();

	return TRUE;
}

void CFluxPartitioningDlg::OnBtnLaod() 
{
	// TODO: ̈ʒuɃRg[ʒmnhp̃R[hǉĂ
	CFileDialog dlg(TRUE, NULL, NULL, 
		OFN_FILEMUSTEXIST| OFN_HIDEREADONLY| OFN_LONGNAMES,
		"Setting  File   (*.set.txt)|*.set.txt|",
		NULL);

	CString strFilePass;
	if(dlg.DoModal() == IDOK){
		strFilePass = dlg.GetPathName();
	}
	else{
		return;
	}

	SetParameterFromFile(strFilePass);
}

void CFluxPartitioningDlg::OnBtnSave() 
{
	CFileDialog dlg(FALSE, NULL, NULL, 
		OFN_FILEMUSTEXIST| OFN_HIDEREADONLY| OFN_LONGNAMES,
		"Setting  File   (*.set.txt)|*.set.txt|",
		NULL);

	CString strFilePass, strLine;
	if(dlg.DoModal() == IDOK){
		strFilePass = dlg.GetPathName();
	}
	else{
		return;
	}

	if(strFilePass != "" && strFilePass.Right(8) != ".set.txt"){
		strFilePass += ".set.txt";
	}

	SaveSetting(strFilePass);
}

void CFluxPartitioningDlg::SetForBatchSetting(const CString& strIniFile, const CString& strInputFile, const CString& strOutputDir, const CString& strFixedFile, const CString& strFileGPP, const CString& strFileRE, const CString& strIniUsterFile)
{
	m_strInputFile = strInputFile;
	m_strInputFileApply = strInputFile;
	m_strOutputFile = strOutputDir;
	m_strOutputFileApply = strOutputDir;
	m_strFixedDataFile = strFixedFile;
	m_strInputGPP = strFileGPP;
	m_strInputRE = strFileRE;
	m_strIniUsterFile = strIniUsterFile;

	if(strIniFile != "") SetParameterFromFile(strIniFile);
	if(m_strIniUsterFile != "") SetParameterFromFile(m_strIniUsterFile);
	UpdateData(FALSE);
}

void CFluxPartitioningDlg::UpdateGraphics()
{
	OnCheckMaxB() ;
	OnCheckFixedData();
	OnCheckNonRectangular() ;
	OnCheckPhotosynthesis() ;
	OnCheckRespiration() ;
	OnCheckMdv() ;
	OnCheckNlr() ;
	OnCheckUseRegression() ;
	OnCheckVanGorsel() ;
}

void CFluxPartitioningDlg::SaveSetting(const CString& strFilePass) 
{
	CString strLine;

	try{
		CStdioFile out(strFilePass, CFile::modeCreate | CFile::modeWrite);

		// Write Setting of Nono-Linear Regression Function parameters
		strLine.Format("PhotosynthesisTerm:%d\n", m_iPhotosynthesisTerm);
		out.WriteString(strLine);

		strLine.Format("RespirationTerm:%d\n", m_iRespirationTerm);
		out.WriteString(strLine);

		strLine.Format("QCThresholdSigma:%lf\n", m_dQCThresholdSigma);
		out.WriteString(strLine);

		strLine.Format("QCMaximumLoop:%d\n", m_iQCMaximumLoop);
		out.WriteString(strLine);

		strLine.Format("SetMaximumB:%d\n", m_bSetMaximumB);
		out.WriteString(strLine);

		strLine.Format("MaximumB:%lf\n", m_dMaximumB);
		out.WriteString(strLine);

		strLine.Format("UseFixedData:%d\n", m_bUseFixedData);
		out.WriteString(strLine);

		strLine.Format("UseNonRectangular:%d\n", m_bUseNonRectangular);
		out.WriteString(strLine);

		strLine.Format("Convexity:%lf\n", m_dConvexity);
		out.WriteString(strLine);

		strLine.Format("UseLT_Equation:%d\n", m_bUseLT_Equation);
		out.WriteString(strLine);

		strLine.Format("MaximumQ10:%lf\n", m_dMaximumQ10);
		out.WriteString(strLine);

		strLine.Format("dReferenceT_RE:%lf\n", m_dReferenceT_RE);
		out.WriteString(strLine);

		strLine.Format("Photosynthesis:%d\n", m_bPhotosynthesis);
		out.WriteString(strLine);

		strLine.Format("Respiration:%d\n", m_bRespiration);
		out.WriteString(strLine);

		strLine.Format("FixedDataFile:%s\n", m_strFixedDataFile);
		out.WriteString(strLine);

		strLine.Format("InputFile:%s\n", m_strInputFile);
		out.WriteString(strLine);

		strLine.Format("OutputFile:%s\n", m_strOutputFile);
		out.WriteString(strLine);

		// Write Setting of Gap-filling parameters
		strLine.Format("GapFillingTerm:%d\n", m_iGapFillingTerm);
		out.WriteString(strLine);

		strLine.Format("MeanDiurnalVariation:%d\n", m_bMeanDiurnalVariation);
		out.WriteString(strLine);

		strLine.Format("NonLinearRegression:%d\n", m_bNonLinearRegression);
		out.WriteString(strLine);

		strLine.Format("UseLinearRegression:%d\n", m_bUseLinearRegression);
		out.WriteString(strLine);

		strLine.Format("MaximumForLinearRegression:%d\n", m_iMaximumForLinearRegression);
		out.WriteString(strLine);

		// Write Setting of Common parameters
		strLine.Format("RunPerDay:%d\n", m_iRunPerDay);
		out.WriteString(strLine);

		strLine.Format("Ustar_threshold:%lf\n", m_dUstar_threshold);
		out.WriteString(strLine);

		strLine.Format("UseVanGorsel:%d\n", m_bUseVanGorsel);
		out.WriteString(strLine);

		strLine.Format("SunsetRunVanGorsel:%d\n", m_iSunsetRunVanGorsel);
		out.WriteString(strLine);

		strLine.Format("NighttimeRadiation:%lf\n", m_dNighttimeRadiation);
		out.WriteString(strLine);

		strLine.Format("UsePeriod:%d\n", m_bUsePeriod);
		out.WriteString(strLine);

		strLine.Format("MovingAverage:%d\n", m_bMovingAverage);
		out.WriteString(strLine);

		strLine.Format("BootstrapNum_uncertainty:%d\n", m_iBootstrapNum);
		out.WriteString(strLine);

		strLine.Format("EliminateNegativeNEE:%d\n", m_bEliminateNegativeNEE);
		out.WriteString(strLine);

		// Write Setting of Apply file name
		strLine.Format("InputGPP:%s\n", m_strInputGPP);
		out.WriteString(strLine);

		strLine.Format("InputRE:%s\n", m_strInputRE);
		out.WriteString(strLine);

		out.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
	}
}

BOOL CFluxPartitioningDlg::SetParameterFromFile(CString strFilePass)
{
	BOOL bOK = TRUE;
	try{
		CStdioFile in(strFilePass, CFile::modeRead | CFile::shareDenyWrite);
		CString strLine, strTemp;
		double dTemp;
		int iTemp;

		while(in.ReadString(strLine)){
			strTemp = strLine.Left(strLine.Find(':'));

			if(strTemp == "PhotosynthesisTerm"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_iPhotosynthesisTerm = iTemp;
				continue;
			}

			if(strTemp == "RespirationTerm"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_iRespirationTerm = iTemp;
				continue;
			}

			if(strTemp == "QCThresholdSigma"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_dQCThresholdSigma = dTemp;
				continue;
			}

			if(strTemp == "QCMaximumLoop"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_iQCMaximumLoop = iTemp;
				continue;
			}

			if(strTemp == "SetMaximumB"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bSetMaximumB = iTemp;
				continue;
			}

			if(strTemp == "MaximumB"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_dMaximumB= dTemp;
				continue;
			}

			if(strTemp == "UseFixedData"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bUseFixedData= iTemp;
				continue;
			}

			if(strTemp == "UseNonRectangular"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bUseNonRectangular= iTemp;
				continue;
			}

			if(strTemp == "Convexity"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_dConvexity = dTemp;
				continue;
			}

			if(strTemp == "UseLT_Equation"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bUseLT_Equation = iTemp;
				if(m_bUseLT_Equation == FALSE){
					m_dMaximumQ10 = 5.0;
				}
				if(m_bUseLT_Equation == TRUE){
					m_dMaximumQ10 = 450.0 * 8.314472;
				}
				continue;
			}

			if(strTemp == "dReferenceT_RE"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_dReferenceT_RE = dTemp;
				continue;
			}

			if(strTemp == "MaximumQ10"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_dMaximumQ10 = dTemp;
				continue;
			}

			if(strTemp == "Photosynthesis"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bPhotosynthesis = iTemp;
				continue;
			}

			if(strTemp == "Respiration"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bRespiration = iTemp;
				continue;
			}

			//if(strTemp == "FixedDataFile"){
			//	strLine = strLine.Mid(strLine.Find(':') + 1);
			//	m_strFixedDataFile = strLine;
			//	continue;
			//}

			//if(strTemp == "InputFile"){
			//	strLine = strLine.Mid(strLine.Find(':') + 1);
			//	m_strInputFile = strLine;
			//	continue;
			//}

			//if(strTemp == "OutputFile"){
			//	strLine = strLine.Mid(strLine.Find(':') + 1);
			//	m_strOutputFile = strLine;
			//	continue;
			//}

			if(strTemp == "GapFillingTerm"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_iGapFillingTerm = iTemp;
				continue;
			}

			if(strTemp == "MeanDiurnalVariation"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bMeanDiurnalVariation = iTemp;
				continue;
			}

			if(strTemp == "NonLinearRegression"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bNonLinearRegression = iTemp;
				continue;
			}

			if(strTemp == "UseLinearRegression"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bUseLinearRegression = iTemp;
				continue;
			}

			if(strTemp == "MaximumForLinearRegression"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_iMaximumForLinearRegression = iTemp;
				continue;
			}

			if(strTemp == "RunPerDay"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_iRunPerDay = iTemp;
				continue;
			}

			if(strTemp == "Ustar_threshold"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_dUstar_threshold = dTemp;
				continue;
			}

			if(strTemp == "UseVanGorsel"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_bUseVanGorsel = dTemp;
				continue;
			}

			if(strTemp == "SunsetRunVanGorsel"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_iSunsetRunVanGorsel = dTemp;
				continue;
			}

			if(strTemp == "NighttimeRadiation"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				dTemp = atof(strLine);
				m_dNighttimeRadiation = dTemp;
				continue;
			}

			if(strTemp == "UsePeriod"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bUsePeriod = iTemp;
				continue;
			}

			if(strTemp == "MovingAverage"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bMovingAverage = iTemp;
				continue;
			}

			if(strTemp == "BootstrapNum_uncertainty"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_iBootstrapNum = iTemp;
				continue;
			}

			if(strTemp == "EliminateNegativeNEE"){
				strLine = strLine.Mid(strLine.Find(':') + 1);
				iTemp = atoi(strLine);
				m_bEliminateNegativeNEE = iTemp;
				continue;
			}
		}

		UpdateData(FALSE);
		UpdateGraphics();

		in.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		bOK = FALSE;
	}

	return bOK;
}
	

void CFluxPartitioningDlg::OnBtnUncertain() 
{
	BOOL bIsDebug = FALSE;
	#ifdef _DEBUG
	bIsDebug = TRUE;
	#endif

	CString strErrorMessage;
	if(bIsDebug){
		if(Determine_uncertainties_bootstraping(strErrorMessage) == TRUE){
			AfxMessageBox("The calculation has been successfully done !");
		}
		else{
			AfxMessageBox(strErrorMessage);
		}
	}
	else{
		// Using Malti Thread
		if(!m_bIsThread){
			m_pThread = AfxBeginThread(OnBtnUncertainThread,(LPVOID)this);
			m_pThread->m_bAutoDelete = FALSE;
			m_pThread->ResumeThread();
		}
		else{
			AfxMessageBox("Error : Thread is busy !");
		}
	}
}

UINT CFluxPartitioningDlg::OnBtnUncertainThread(LPVOID pParam)
{
	CFluxPartitioningDlg* pDlg = static_cast<CFluxPartitioningDlg*>(pParam);

 	pDlg->m_bIsThread = TRUE;
	if(pDlg->Determine_uncertainties_bootstraping(pDlg->m_strErrorMessageForThread) == TRUE){
		AfxMessageBox("The calculation has been successfully done !");
	}
	else{
		if(pDlg->m_strErrorMessageForThread != ""){
			AfxMessageBox(pDlg->m_strErrorMessageForThread);
		}
	}

	pDlg->m_bIsThread = FALSE;
	return 0;
}

BOOL CFluxPartitioningDlg::Determine_uncertainties_bootstraping(CString& strErrorMessage) 
{
	// TODO: ̈ʒuɃRg[ʒmnhp̃R[hǉĂ
	UpdateData(TRUE);

	BeginWaitCursor();

	///////////////////////////////////////////
	// eLXg{bNX̓͂̃`FbN
	///////////////////////////////////////////
	if(ErrorCheck(strErrorMessage) == FALSE){
		return FALSE;
	}
	m_strInputFileApply = m_strInputFile;
	m_strOutputFileApply = m_strInputFileApply.Mid(0, m_strInputFileApply.ReverseFind('\\'));
	UpdateData(FALSE);

	CString strMovingBlock = "mv";
	if(m_bMovingAverage == TRUE){
		strMovingBlock.Format("mv%d", m_iBootstrapNum);
	}
	else{
		strMovingBlock.Format("bk%d", m_iBootstrapNum);
	}

	// ***************************************************************
	// ݒt@C̏o
	// ***************************************************************
	CString strOutSettingFile = m_strInputFile;
	strOutSettingFile = strOutSettingFile.Left(strOutSettingFile.ReverseFind('.')) + ".uncertainties.set.txt";
	strOutSettingFile = m_strOutputFile + "\\" + strOutSettingFile.Mid(strOutSettingFile.ReverseFind('\\') + 1);
	SaveSetting(strOutSettingFile);

	///////////////////////////////////////////
	// Œp^̓ǂݍ
	///////////////////////////////////////////	
	if(m_bUseFixedData){
		if(ReadFixedParamterFile() == FALSE) return FALSE;
	}

	// **************************************************
	// vOXo[̏
	// **************************************************
	const int iProgressRange = 11000;
	m_ctlProgress.SetRange(0, iProgressRange);	// Inisilize Progress bar
	m_ctlProgress.SetPos(100);

	///////////////////////////////////////////
	// ̓t@C̓ǂݍ
	///////////////////////////////////////////	
	vector< vector<CPhotosynthesisParameters> > aryBootstrapPhoto;
	vector< vector<CRespirationParameters> > aryBootstrapResp;
	if(m_bNonLinearRegression){
		vector<CMeanTerm*> aryMeanTermGPP;
		vector<CMeanTerm*> aryMeanTermRE;

		CString strHeaderLine;
		BOOL bExistHeader = TRUE;
		if(m_bMovingAverage){
			if(ReadFile_with_MovingWindow(m_strInputFile, aryMeanTermGPP, aryMeanTermRE, m_iRunPerDay, m_iPhotosynthesisTerm, m_iRespirationTerm, strHeaderLine) == FALSE){
				// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
				strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
				return FALSE;
			}
		}
		else{
			if(ReadFile_with_BlockWindow(m_strInputFile, aryMeanTermGPP, aryMeanTermRE, m_iRunPerDay, m_iPhotosynthesisTerm, m_iRespirationTerm, strHeaderLine) == FALSE){
				// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
				strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
				return FALSE;
			}
		}
		m_ctlProgress.SetPos(1000 / 2);
		m_bBootstraping = TRUE;
	
		///////////////////////////////////////////
		// ֘Ãp^Zo
		///////////////////////////////////////////	
		if(m_bPhotosynthesis == TRUE){
			DeterminePhotosynthesisParameters(aryMeanTermGPP, strHeaderLine, m_ctlProgress);
			for(int i = 0; i < aryMeanTermGPP.size(); i++){
				vector<CPhotosynthesisParameters> cPhoto;
				aryMeanTermGPP[i]->GetBootstrapPhotsynthesisParameters(cPhoto);
				aryBootstrapPhoto.push_back(cPhoto);
			}
		}
		else{
			// mۂ̈̊J
			for(int i = 0; i < aryMeanTermGPP.size(); i++){
				aryMeanTermGPP[i]->ReleaseDataset();
				delete aryMeanTermGPP[i];
			}
		}
		// mۂ̈̊J
		for(int i = 0; i < aryMeanTermGPP.size(); i++){
			aryMeanTermGPP[i]->ReleaseDataset();
			delete aryMeanTermGPP[i];
		}
		aryMeanTermGPP.clear();

		///////////////////////////////////////////
		// ċz֘Ãp^Zo
		///////////////////////////////////////////	
		if(m_bRespiration == TRUE){
			DetermineRespirationParameters(aryMeanTermRE, strHeaderLine, m_ctlProgress);
			for(int i = 0; i < aryMeanTermRE.size(); i++){
				vector<CRespirationParameters> cResp;
				aryMeanTermRE[i]->GetBootstrapRespirationParameters(cResp);
				aryBootstrapResp.push_back(cResp);
			}
		}
		else{
			// mۂ̈̊J
			for(int i = 0; i < aryMeanTermRE.size(); i++){
				aryMeanTermRE[i]->ReleaseDataset();
				delete aryMeanTermRE[i];
			}
		}
		// mۂ̈̊J
		for(i = 0; i < aryMeanTermRE.size(); i++){
			aryMeanTermRE[i]->ReleaseDataset();
			delete aryMeanTermRE[i];
		}
		aryMeanTermRE.clear();
	}
	m_ctlProgress.SetPos(iProgressRange / 2);

	///******************************************************************************
	// ϓω@@Ɓ@\@ɂ⊮ 
	///******************************************************************************
	CString strHeader;
	vector<CMeanTerm*> aryMeanTermGap;
	if(m_bMeanDiurnalVariation == TRUE){
		if(m_bMovingAverage){
			if(ReadFile_with_MovingWindow(m_strInputFileApply, aryMeanTermGap, m_iRunPerDay, m_iGapFillingTerm, strHeader) == FALSE){
				// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
				strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
				return FALSE;
			}
		}
		else{
			if(ReadFile_with_BlockWindow(m_strInputFileApply, aryMeanTermGap, m_iRunPerDay, m_iGapFillingTerm, strHeader) == FALSE){
				// AfxMessageBox ("̓t@C̓ǂݍ݂Ɏs܂B");
				strErrorMessage = "̓t@C̓ǂݍ݂Ɏs܂B";
				return FALSE;
			}
		}

		if(GapFill_mdv_lt(strHeader, aryMeanTermGap, m_bMovingAverage, m_ctlProgress) == FALSE){
			// mۂ̈̊J
			for(int i = 0; i < aryMeanTermGap.size(); i++){
				aryMeanTermGap[i]->ReleaseDataset();
				delete aryMeanTermGap[i];
			}
			return FALSE;
		}
	}
	m_ctlProgress.SetPos(iProgressRange / 2 + 1000);

	if(m_bMovingAverage == FALSE){
		return TRUE;
	}

	// when van Gorsel filtering is applied
	if(m_bUseVanGorsel) strMovingBlock += ".vG";

	///******************************************************************************
	// t@C̏ó@ibootstrap ρEW΍, smj
	///******************************************************************************
	CString strOutFilledFinalFile = m_strInputFile;
	strOutFilledFinalFile = strOutFilledFinalFile.Left(strOutFilledFinalFile.ReverseFind('.'))  + "." + strMovingBlock +  ".final_filled_bootstrap.csv";
	strOutFilledFinalFile = m_strOutputFile + "\\" + strOutFilledFinalFile.Mid(strOutFilledFinalFile.ReverseFind('\\') + 1);
	CStdioFile file_filled(strOutFilledFinalFile, CFile::modeCreate | CFile::modeWrite);

	// wb_[Lq
	CString strLine;
	strLine.Format("Date, H_filled, LE_filled, NEE_filled, GPP_filled, RE_filled, H_stdev, LE_stdev, NEE_stdev, GPP_stdev, RE_stdev\n");
	file_filled.WriteString(strLine);
	strLine.Format(", W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1\n");
	file_filled.WriteString(strLine);

	///******************************************************************************
	// sm̏o i30lAʁANԁj
	///******************************************************************************
	CString strDayUncertaintyFile = m_strInputFile;
	strDayUncertaintyFile = strDayUncertaintyFile.Left(strDayUncertaintyFile.ReverseFind('.')) + "." + strMovingBlock + ".final_uncertainties_daily_bootstrap.csv";
	strDayUncertaintyFile = m_strOutputFile + "\\" + strDayUncertaintyFile.Mid(strDayUncertaintyFile.ReverseFind('\\') + 1);

	CStdioFile file_uncertainty1(strDayUncertaintyFile, CFile::modeCreate | CFile::modeWrite);

	// wb_[Lq
	strLine.Format("Date, H_filled, LE_filled, NEE_filled, GPP_filled, RE_filled, H_stdev, LE_stdev, NEE_stdev, GPP_stdev, RE_stdev\n");
	file_uncertainty1.WriteString(strLine);

	strLine.Format(", W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1\n");
	file_uncertainty1.WriteString(strLine);

	vector<double> aryAnnualH, aryAnnualLE, aryAnnualNEE, aryAnnualGPP, aryAnnualRE;
	vector<int> aryAnnualH_num, aryAnnualLE_num, aryAnnualNEE_num, aryAnnualGPP_num, aryAnnualRE_num;

	///******************************************************************************
	// Non-Linear Regression method start 
	///******************************************************************************
	int iUstar = 0;
	int iHeatFlux = 1;
	int iLatentFlux = 2;
	int iNEEraw = 3;
	int iRawRadiationColumn = 4;
	int iRadiationColumn = 7;
	int iTemperatureColumn = 8;
	int iVPDColumn = 9;
	int iPeriodColumn = 10;
	int iNEE_model_nlr1 = -1;
	int iRE = -1;
	int iNEE_nlr1 = -1;
	int iNEE_nlr2 = -1;
	int iNEE_model = -1;
	int iGPP = -1;
	int iGPP2 = -1;
	int iRE2 = -1;

	if(m_bNonLinearRegression){
		// t@CEtH_쐬Aʂۑ
		vector<int> aryOutColumnNum;
		BOOL bUseNegativeValues = TRUE;

		int iProgressBar = iProgressRange - (iProgressRange / 2 + 1000) - 2000;
		iProgressBar /= aryMeanTermGap.size();
		
		for(int i = 0; i < aryMeanTermGap.size(); i++){	
			aryMeanTermGap[i]->SetPhotosynthesisConstant(bUseNegativeValues, m_bUseNonRectangular, m_dConvexity);
			aryMeanTermGap[i]->SetRespirationConstant(bUseNegativeValues, m_bUseLT_Equation, m_dUstar_threshold, m_dNighttimeRadiation);

			// bootstrap non-linear parameters NXɃRs[
			if(aryMeanTermGap[i]->SetBootstrapNonLinearParameters(aryBootstrapPhoto[i], aryBootstrapResp[i]) == FALSE){
				strErrorMessage = "Fail in copying bootstrap parameters for non-linear regresson.";
				return FALSE;
			}

			// poȐxvZ
			if((iNEE_model_nlr1 = aryMeanTermGap[i]->ApplyLightPhotosynthesisCurve_bootstrap(iRadiationColumn)) <= 0 ){
				strErrorMessage = "Fail in calcularing NEE by non-linear regression1.";
				return FALSE;
			}
			iNEE_model_nlr1 -= 1;

			// Ԍnċzʂ̌vZ (RE)
			if((iRE = aryMeanTermGap[i]->ApplyRespirationCurve_bootstrap(iTemperatureColumn)) <= 0 ){
				strErrorMessage = "Fail in calcularing respiration.";
				return FALSE;
			}
			iRE -= 1;

			// `NEE ⊮@i[Ȑƌċzgpꍇj
			if((iNEE_nlr2 = aryMeanTermGap[i]->GapFill_NEE_NonLinearRegression_bootstrap(iNEEraw, iUstar, iRadiationColumn,  iNEE_model_nlr1, iRE, m_iBootstrapNum)) <= 0 ){
				strErrorMessage = "Fail in gap-filling NEE by using NEE by non-linear regression2.";
				return FALSE;
			}
			iNEE_nlr2 -= 1;

			// `NEE 𐄒@i[ȐƌċzɂNEE ̐j
			if((iNEE_model = aryMeanTermGap[i]->Estimate_NEE_NonLinearRegression_bootstrap(iRadiationColumn,  iNEE_model_nlr1, iRE, m_iBootstrapNum)) <= 0 ){
				strErrorMessage = "Fail in estimating NEE by using NEE by non-linear regression1.";
				return FALSE;
			}
			iNEE_model -= 1;

			// GPP @iGPP ̐j
			if((iGPP = aryMeanTermGap[i]->Estimate_GPP_bootstrap(iNEE_nlr2,  iRE, m_iBootstrapNum)) <= 0 ){
				strErrorMessage = "Fail in estimating GPP.";
				return FALSE;
			}
			iGPP -= 1;

			if((iGPP2 = aryMeanTermGap[i]->Adjust_GPP_RE_bootstrap(iGPP,  iRE, iGPP2, iRE2, m_iBootstrapNum)) <= 0 ){
				strErrorMessage = "Fail in estimating GPP.";
				return FALSE;
			}
			iGPP2 -= 1;
			iRE2 -= 1;

			///******************************************************************************
			// LUT, NLR, MDV @ɂ錇f[^̍ŏIIȕ⊮sȂ
			// D揇ʂ́A1. Linear Regression, 2. LUT, 3. NLR, 4.MDV
			// Boot strap gap-filling l̕ϒlƕW΍Zo
			///******************************************************************************
			BOOL bNEE = FALSE;
			aryMeanTermGap[i]->Final_Priority_bootstrap(iHeatFlux, m_iBootstrapNum, m_iMaximumForLinearRegression, m_bMovingAverage, bNEE);
			aryMeanTermGap[i]->Final_Priority_bootstrap(iLatentFlux, m_iBootstrapNum, m_iMaximumForLinearRegression, m_bMovingAverage, bNEE);
			bNEE = TRUE;
			aryMeanTermGap[i]->Final_Priority_bootstrap(iNEEraw, m_iBootstrapNum, m_iMaximumForLinearRegression, m_bMovingAverage, bNEE, aryMeanTermGap[i]->GetDataset(iNEE_model));
			aryMeanTermGap[i]->Calculate_GPP_bootstrap(aryMeanTermGap[i]->GetDataset(iNEEraw), aryMeanTermGap[i]->GetDataset(iRE2), aryMeanTermGap[i]->GetDataset(iGPP2), m_iBootstrapNum, m_bMovingAverage);

			///******************************************************************************
			// t@C̏ó@ibootstrap ρEW΍, smj
			///******************************************************************************
			try{
				if(aryMeanTermGap[i]->WriteStringBoostrapFinal(file_filled, iHeatFlux, iLatentFlux, iNEEraw, iGPP2, iRE2, m_bMovingAverage) == FALSE){
					strErrorMessage = "Fail in outputting the final file.";
					return FALSE;
				}			
			}
			catch(CException* e){
				e->ReportError();
				e->Delete();
				return FALSE;
			}

			///******************************************************************************
			// t@C̏ó@ibootstrap ρEW΍, smj
			///******************************************************************************
			// ʒlEsm萫̌vZAo
			if(aryMeanTermGap[i]->WriteStringUncertaintyDay(file_uncertainty1, iHeatFlux, iLatentFlux, iNEEraw, iGPP2, iRE2, m_bMovingAverage) == FALSE){
				strErrorMessage = "Fail in outputting the daily uncertainty file.";
				return FALSE;
			}

			for(int iBootstrap = 0; iBootstrap < m_iBootstrapNum; iBootstrap++){
				if(i == 0){
					aryAnnualH.push_back(0.0);
					aryAnnualLE.push_back(0.0);
					aryAnnualNEE.push_back(0.0);
					aryAnnualGPP.push_back(0.0);
					aryAnnualRE.push_back(0.0);

					aryAnnualH_num.push_back(0);
					aryAnnualLE_num.push_back(0);
					aryAnnualNEE_num.push_back(0);
					aryAnnualGPP_num.push_back(0);
					aryAnnualRE_num.push_back(0);
				}

				aryMeanTermGap[i]->ForBootstrapAnnualMeanStdev(iHeatFlux, aryAnnualH[iBootstrap], aryAnnualH_num[iBootstrap], iBootstrap);
				aryMeanTermGap[i]->ForBootstrapAnnualMeanStdev(iLatentFlux, aryAnnualLE[iBootstrap], aryAnnualLE_num[iBootstrap], iBootstrap);
				aryMeanTermGap[i]->ForBootstrapAnnualMeanStdev(iNEEraw, aryAnnualNEE[iBootstrap], aryAnnualNEE_num[iBootstrap], iBootstrap);
				aryMeanTermGap[i]->ForBootstrapAnnualMeanStdev(iGPP2, aryAnnualGPP[iBootstrap], aryAnnualGPP_num[iBootstrap], iBootstrap);
				aryMeanTermGap[i]->ForBootstrapAnnualMeanStdev(iRE2, aryAnnualRE[iBootstrap], aryAnnualRE_num[iBootstrap], iBootstrap);
			}

			///******************************************************************************
			// ̊J
			///******************************************************************************
			aryMeanTermGap[i]->ReleaseDataset();
			delete aryMeanTermGap[i];

			m_ctlProgress.SetPos(iProgressRange / 2 + 1000 + iProgressBar * i);
		}
		aryBootstrapPhoto.clear();
		aryBootstrapResp.clear();
	}

	file_filled.Close();

	///******************************************************************************
	// sm̏o iNԁj
	///******************************************************************************
	CString strAnnualUncertaintyFile = m_strInputFile;
	strAnnualUncertaintyFile = strAnnualUncertaintyFile.Left(strAnnualUncertaintyFile.ReverseFind('.')) + "." + strMovingBlock +  ".final_uncertainties_annual_bootstrap.csv";
	strAnnualUncertaintyFile = m_strOutputFile + "\\" + strAnnualUncertaintyFile.Mid(strAnnualUncertaintyFile.ReverseFind('\\') + 1);

	CStdioFile file_uncertainty2(strAnnualUncertaintyFile, CFile::modeCreate | CFile::modeWrite);

	strLine.Format("Date, H_filled, LE_filled, NEE_filled, GPP_filled, RE_filled, H_stdev, LE_stdev, NEE_stdev, GPP_stdev, RE_stdev\n");
	file_uncertainty2.WriteString(strLine);

	strLine.Format(", W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1, W m-2, W m-2, umol m-2 s-1, umol m-2 s-1, umol m-2 s-1\n");
	file_uncertainty2.WriteString(strLine);

	try{
		// NԒlEsm萫Ɋւϐ
		double dAnnualHeatFlux = 0.0, dAnnualLatentFlux = 0.0, dAnnualNEE = 0.0, dAnnualGPP = 0.0, dAnnualRE = 0.0;
		double dAnnualHeatFlux_stdev = 0.0, dAnnualLatentFlux_stdev = 0.0, dAnnualNEE_stdev = 0.0, dAnnualGPP_stdev = 0.0, dAnnualRE_stdev = 0.0;
		double dTx_HeatFlux = 0.0, dTx_LatentFlux = 0.0, dTx_NEE = 0.0, dTx_GPP = 0.0, dTx_RE = 0.0;
		double dTxx_HeatFlux = 0.0, dTxx_LatentFlux = 0.0, dTxx_NEE = 0.0, dTxx_GPP = 0.0, dTxx_RE = 0.0;
		int iHeatFluxNum = 0, iLatentFluxNum = 0, iNEENum = 0, iGPPNum = 0, iRENum = 0;

		for(int iBootstrap = 0; iBootstrap < m_iBootstrapNum; iBootstrap++){
			dTx_HeatFlux = aryAnnualH[iBootstrap];
			dTx_LatentFlux = aryAnnualLE[iBootstrap];
			dTx_NEE = aryAnnualNEE[iBootstrap];
			dTx_GPP = aryAnnualGPP[iBootstrap];
			dTx_RE = aryAnnualRE[iBootstrap];

			iHeatFluxNum = aryAnnualH_num[iBootstrap];
			iLatentFluxNum = aryAnnualLE_num[iBootstrap];
			iNEENum = aryAnnualNEE_num[iBootstrap];
			iGPPNum = aryAnnualGPP_num[iBootstrap];
			iRENum = aryAnnualRE_num[iBootstrap];


			dAnnualHeatFlux += dTx_HeatFlux / iHeatFluxNum;
			dTxx_HeatFlux += (dTx_HeatFlux / iHeatFluxNum) * (dTx_HeatFlux / iHeatFluxNum);
			dAnnualLatentFlux += dTx_LatentFlux / iLatentFluxNum;
			dTxx_LatentFlux += (dTx_LatentFlux / iLatentFluxNum) * (dTx_LatentFlux / iLatentFluxNum);
			dAnnualNEE += dTx_NEE / iNEENum;
			dTxx_NEE += (dTx_NEE / iNEENum) * (dTx_NEE / iNEENum);
			dAnnualGPP += dTx_GPP / iGPPNum;
			dTxx_GPP += (dTx_GPP / iGPPNum) * (dTx_GPP / iGPPNum);
			dAnnualRE += dTx_RE / iRENum;
			dTxx_RE += (dTx_RE / iRENum) * (dTx_RE / iRENum);

			dTx_HeatFlux = 0.0;
			iHeatFluxNum = 0;
			dTx_LatentFlux = 0.0;
			iLatentFluxNum = 0;
			dTx_NEE = 0.0;
			iNEENum = 0;
			dTx_GPP = 0.0;
			iGPPNum = 0;
			dTx_RE = 0.0;
			iRENum = 0;
		}

		dAnnualHeatFlux_stdev = sqrt( (dTxx_HeatFlux - dAnnualHeatFlux * dAnnualHeatFlux / m_iBootstrapNum) / m_iBootstrapNum );
		dAnnualHeatFlux = dAnnualHeatFlux / m_iBootstrapNum;		
		dAnnualLatentFlux_stdev = sqrt( (dTxx_LatentFlux - dAnnualLatentFlux * dAnnualLatentFlux / m_iBootstrapNum) / m_iBootstrapNum );
		dAnnualLatentFlux = dAnnualLatentFlux / m_iBootstrapNum;		
		dAnnualNEE_stdev = sqrt( (dTxx_NEE - dAnnualNEE * dAnnualNEE / m_iBootstrapNum) / m_iBootstrapNum );
		dAnnualNEE = dAnnualNEE / m_iBootstrapNum;		
		dAnnualGPP_stdev = sqrt( (dTxx_GPP - dAnnualGPP * dAnnualGPP / m_iBootstrapNum) / m_iBootstrapNum );
		dAnnualGPP = dAnnualGPP / m_iBootstrapNum;	
		dAnnualRE_stdev = sqrt( (dTxx_RE - dAnnualRE * dAnnualRE / m_iBootstrapNum) / m_iBootstrapNum );
		dAnnualRE = dAnnualRE / m_iBootstrapNum;
		
		strLine.Format("Annual, %lf, %lf, %lf, %lf, %lf, %lf, %lf, %lf, %lf, %lf\n", 
				dAnnualHeatFlux, dAnnualLatentFlux, dAnnualNEE, dAnnualGPP, dAnnualRE, 
				dAnnualHeatFlux_stdev, dAnnualLatentFlux_stdev, dAnnualNEE_stdev, dAnnualGPP_stdev, dAnnualRE_stdev);
		file_uncertainty2.WriteString(strLine);

		file_uncertainty1.Close();
		file_uncertainty2.Close();
	}
	catch(CException* e){
		e->ReportError();
		e->Delete();
		return FALSE;
	}

	m_ctlProgress.SetPos(iProgressRange);

	strErrorMessage = "The calculation has been successfully done!";
	m_bBootstraping = FALSE;

	return TRUE;
}
